#ifndef _TEXT_LINE_H_
#define _TEXT_LINE_H_

#include <string>
#include <map>
#include <vector>
#include <glm/glm.hpp>
#include <rag/Image.h>
#include "ITextFont.h"

typedef unsigned int	u32;
typedef signed int		s32;
typedef unsigned short	u16;
typedef signed short	s16;
typedef unsigned char	u8;
typedef signed char		s8;

namespace rag
{
	/// Font system based on bitmap fonts.
    /**
		This kind of fonts assumes the output generated with the AngelCode tool that 
		can be found here: http://www.angelcode.com/products/bmfont/
     */
	class BMPFont: public ITextFont
	{
	public:

		/**
		 * @param path The path where the font descriptor file is. i.e., "assets/"
		 * @param name The font descriptor file name.
		 * It is assumed that font descriptor and font texture are in the same folder.
		 */
		BMPFont(const std::string& path, const std::string& name);
		virtual ~BMPFont();

		
		
		virtual int     getWidth(const std::string& text); ///< Returns the width of a text
        virtual void    print(const std::string& text, const glm::mat4& matrix); ///< Renders text. Asumes ortho projection 1:1

	  private:

		  struct Glyph
		  {
			  u32 id;
			  u16 x, y;
			  u16 width, height;
			  s16 xoffset, yoffset;
			  u16 xadvance;
			  u8  page;
			  u8  chnl;
		  };

		  class FontBinDescriptorReader
		  {

		  public:

			  /**
			  Reads a font descriptor and returns glyphs.
			  */
			  std::map<int, Glyph*> read(std::string fontDescriptorFile, int* outHeight, std::string& outTexturePath);

		  private:

			  std::map<int, Glyph*> mGlyphs;

		  };

		void printGlyph(Glyph* glyph);

		/// Returns a vector with utf8 decoded values
		int* decodeText(const char* text, int* size);

		/// Decodes a utf8 encoded character
		int decodeUTF8(const char *encodedBuffer, unsigned int *outLength);

		/// Returns width faster than getWidth()
		int getDecodedStringWidth(int* decodedString, int size);

		std::map<int, Glyph*> mGlyphs;
		
		int mHeight; ///< height of the higher glyph
		int mDecodedText[1024];

		Image* image;

	};

}

#endif