#include "Bitmap.h"
#include <rag/Platform.h>
#include <rag/Screen.h>
#include <rag/ResourceMgr.h>
#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp>
#include <glm/gtc/type_ptr.hpp>
#include <rag/Renderer.h>

using namespace rag;

Bitmap::Bitmap(const std::string& path):
uv(0.0f, 1.0f, 0.0f, 1.0f)
{
	image = Image::loadImage(path);
	image->acquire();
	material.textureId = image->name;
    width = image->width;
    height = image->height;
	material.vsh = "assets/shaders/Tint.vsh";
	material.fsh = "assets/shaders/Tint.fsh";
}

Bitmap::Bitmap(Image* image):
uv(0.0f, 1.0f, 0.0f, 1.0f)
{
    this->image = image;
	image->acquire();
	material.textureId = image->name;
    width = image->width;
    height = image->height;
	material.vsh = "assets/shaders/Tint.vsh";
	material.fsh = "assets/shaders/Tint.fsh";
}

Bitmap::~Bitmap()
{
	image->release();
}

void Bitmap::render()
{
    using namespace glm;

	if (image->state != Resource::Ready)
		return;

	renderMaterial->textureId = material.textureId = image->name; // TODO some way to change material without messing up textureId.
    
	static rag::VertexArray vertexArray;
	vertexArray.clear();

	rag::Vertex vertices[4];
	vertices[0].position.x = 0;
	vertices[0].position.y = height;
	vertices[1].position.x = width;
	vertices[1].position.y = height;
	vertices[2].position.x = width;
	vertices[2].position.y = 0;
	vertices[3].position.x = 0;
	vertices[3].position.y = 0;

    updateMatrix();
    for (int i = 0; i < 4; i++)
    {
		rag::Vertex& vertex = vertices[i];
        vec4 t = matrix * vec4(vertex.position, 0, 1);
        vertex.position.x = t[0];
        vertex.position.y = t[1];
    }
    
    // Get bounds and clip to screen
    float mxx = max(max(max(vertices[0].position.x, vertices[1].position.x), vertices[2].position.x), vertices[3].position.x);
	float mnx = min(min(min(vertices[0].position.x, vertices[1].position.x), vertices[2].position.x), vertices[3].position.x);
	float mxy = max(max(max(vertices[0].position.y, vertices[1].position.y), vertices[2].position.y), vertices[3].position.y);
	float mny = min(min(min(vertices[0].position.y, vertices[1].position.y), vertices[2].position.y), vertices[3].position.y);
    rag::Rectangle r(mnx, mny, mxx-mnx, mxy-mny);
    rag::Rectangle screen(0, 0, Screen::realWidth, Screen::realHeight);
    if (!screen.intersects(r))
        return;

	// Color
	for (int i = 0; i < 4; ++i)
	{
		rag::Vertex& vertex = vertices[i];
		vertex.color.r = static_cast<unsigned char>(colorTransform.r * 255);
		vertex.color.g = static_cast<unsigned char>(colorTransform.g * 255);
		vertex.color.b = static_cast<unsigned char>(colorTransform.b * 255);
		vertex.color.a = static_cast<unsigned char>(colorTransform.a * 255);
	}

	// Texture coordinates.
	if (uv.s < uv.t)
	{
		vertices[0].texCoords.x = uv.s;
		vertices[0].texCoords.y = uv.q;
		vertices[1].texCoords.x = uv.t;
		vertices[1].texCoords.y = uv.q;
		vertices[2].texCoords.x = uv.t;
		vertices[2].texCoords.y = uv.p;
		vertices[3].texCoords.x = uv.s;
		vertices[3].texCoords.y = uv.p;
	}
	else
	{
		vertices[0].texCoords.x = uv.t;
		vertices[0].texCoords.y = uv.p;
		vertices[1].texCoords.x = uv.t;
		vertices[1].texCoords.y = uv.q;
		vertices[2].texCoords.x = uv.s;
		vertices[2].texCoords.y = uv.q;
		vertices[3].texCoords.x = uv.s;
		vertices[3].texCoords.y = uv.p;
	}

	vertexArray.push_back(vertices[0]);
	vertexArray.push_back(vertices[1]);
	vertexArray.push_back(vertices[2]);
	vertexArray.push_back(vertices[0]);
	vertexArray.push_back(vertices[2]);
	vertexArray.push_back(vertices[3]);

	rag::RenderTarget& renderTarget = Renderer::getInstance().getRenderTarget();
	renderTarget.draw(vertexArray, *renderMaterial);
}
