#ifndef _RAG_BITMAP_H_
#define _RAG_BITMAP_H_

#include "DisplayObject.h"
#include <rag/Platform.h>
#include <rag/Image.h>

namespace rag
{

	/// Provides the ability to show images.
	/** A Bitmap can be used with a path to an image or directly with an image.
	The Bitmap will represent an arbitrary image and supports all basic transformations as any DisplayObject,
	such as scale, rotation, skew. All those transformations can be inherited in the Display List hierarchy.
	*/
	class Bitmap : public DisplayObject
	{
	public:

		/// Create a Bitmap using an image in a given path.
		/** It is asumed the image exists in the path, otherwise an error is logged,
		and nothing is shown.
		@see Image
		*/
		Bitmap(const std::string& path);

		/// Create a Bitmap with an existing image.
		/** The Bitmap is created with an already existant Image, so images downloaded from The Internet
		or procedurally generated images can be used.
		*/
		Bitmap(Image* image);

		virtual ~Bitmap();

		virtual void render() override;
		virtual void prerender() override {};
        
		/// Shared image.
        Image* image;

		/// Texture coordinates. used with texture atlases.
		/** Will default to (0, 1), (0, 1) coordinates using the entire image, 
			but can be set to any pair of coordinates to use like a sprite
			inside an image atlas. 
		*/
		glm::vec4 uv;
	};

} // rag

#endif
