#include "Color.h"
#include <sstream>
#include <iomanip>
#include <stdlib.h>

using namespace std;
using namespace rag;

// TODO Add constants colors like white, red, blue, light_grey, ...

const Color Color::black(0, 0, 0);
const Color Color::white(1, 1, 1);

Color::Color(float r, float g, float b, float a)
{
    this->r = r;
    this->g = g;
    this->b = b;
    this->a = a;
}

Color::Color(std::string color):
r(1),
g(1),
b(1),
a(1)
{
    size_t found = color.find_first_of("#xX");
    if (found != string::npos)
    {
        color = color.substr(found+1);
    }
    if (color.length() < 6)
	{
		// Fill with zeroes
		while(color.length() < 6)
			color = "0" + color;
	}
     
    r = (float)(strtol(color.substr(0, 2).c_str(), NULL, 16)) / 255;
    g = (float)(strtol(color.substr(2, 2).c_str(), NULL, 16)) / 255;
    b = (float)(strtol(color.substr(4, 2).c_str(), NULL, 16)) / 255;    

	if (color.length() == 8)
	{
		a = (float)(strtol(color.substr(6, 2).c_str(), NULL, 16)) / 255;
	}
}

Color& Color::operator*=(const Color &rhs) 
{
    r *= rhs.r;
    g *= rhs.g;
    b *= rhs.b;
    a *= rhs.a;
    return *this;
}

Color& Color::operator*=(float value) 
{
    r *= value;
    g *= value;
    b *= value;
    a *= value;
    return *this;
}

Color& Color::operator/=(const Color &rhs) 
{
    if (rhs.r != 0) r /= rhs.r;
    if (rhs.g != 0) g /= rhs.g;
    if (rhs.b != 0) b /= rhs.b;
    if (rhs.a != 0) a /= rhs.a;
    return *this;
}

Color& Color::operator/=(float value) 
{
    if (value == 0) return *this;
    
    r /= value;
    g /= value;
    b /= value;
    a /= value;
    return *this;
}

Color& Color::operator+=(const Color &rhs) 
{
    r += rhs.r;
    g += rhs.g;
    b += rhs.b;
    a += rhs.a;
    return *this;
}

bool Color::operator==(const Color &rhs)
{
    return rhs.r == this->r && rhs.g == this->g && rhs.b == this->b && rhs.a == this->a;
}

bool Color::operator!=(const Color &rhs)
{
    return !(*this == rhs);
}

const Color Color::operator*(const Color &rhs) const{
    Color result = *this;
    result *= rhs;
    return result;
}

const Color Color::operator*(float value) const{
    Color result = *this;
    result *= value;
    return result;
}

const Color Color::operator/(const Color &rhs) const{
    Color result = *this;
    result /= rhs;
    return result;
}

const Color Color::operator/(float value) const{
    Color result = *this;
    result /= value;
    return result;
}

const Color Color::operator+(const Color &rhs) const{
    Color result = *this;
    result += rhs;
    return result;
}

std::string Color::toString()
{
	std::stringstream stream;
	stream << std::setfill ('0') << std::setw(2) << std::hex << (int)(r * 255);
	stream << std::setfill ('0') << std::setw(2) << std::hex << (int)(g * 255);
	stream << std::setfill ('0') << std::setw(2) << std::hex << (int)(b * 255);
	stream << std::setfill ('0') << std::setw(2) << std::hex << (int)(a * 255);
	return stream.str();
}

unsigned int Color::toRGBA() const
{
    return createRGBA((unsigned int)(r*255.f), (unsigned int)(g*255.f), (unsigned int)(b*255.f), (unsigned int)(a*255.f));
}

unsigned int Color::toABGR() const
{
    return createABGR((unsigned int)(r*255.f), (unsigned int)(g*255.f), (unsigned int)(b*255.f), (unsigned int)(a*255.f));
}

unsigned int Color::createRGBA(int r, int g, int b, int a)
{
	 return ((r & 0xff) << 24) + ((g & 0xff) << 16) + ((b & 0xff) << 8) + (a & 0xff);
}

unsigned int Color::createABGR(int r, int g, int b, int a)
{
	 return (r & 0xff) + ((g & 0xff) << 8) + ((b & 0xff) << 16) + ((a & 0xff) << 24);
}

