#ifndef Rag_Color_h
#define Rag_Color_h

#include <string>

namespace rag
{

	/// Represents RGBA color.
	/** Provides functionality to operate with colors. */
	class Color
	{
	public:

		/// Construts a Color object default to white.
		Color(float r = 1, float g = 1, float b = 1, float a = 1);

		/// Construts a Color object with a string representing the color in hexadecimal.
		/** The expected format of the string is like [x|#]RRGGBB[AA]. */
		Color(std::string color);

		/// Writes the color in a single 32 bit int.
		unsigned int toRGBA() const;

		/// Writes the color in a single 32 bit int reversed.
		unsigned int toABGR() const;

		/// Returns an unsigned int from color values.
		static unsigned int createRGBA(int r, int g, int b, int a);

		/// Returns an unsigned int from color values.
		static unsigned int createABGR(int r, int g, int b, int a);

		Color& operator*=(const Color &rhs);
		Color& operator*=(float value);
		Color& operator/=(const Color &rhs);
		Color& operator/=(float value);
		Color& operator+=(const Color &rhs);

		const Color operator*(const Color &rhs) const;
		const Color operator*(float value) const;
		const Color operator/(const Color &rhs) const;
		const Color operator/(float value) const;
		const Color operator+(const Color &rhs) const;

		bool operator==(const Color &rhs);
		bool operator!=(const Color &rhs);

		/// Returns a string representation of the Color object.
		std::string toString();

		float r;
		float g;
		float b;
		float a;

		const static Color black;
		const static Color white;
	};

} // rag

#endif
