/* Title: Ease equations library.

   Based on previous work by Robert Penner, Jesus Gollonet and Tomas Cepeda.

   Please check readme for licensing and other conditions.

	//#warning complete with http://snippets.dzone.com/posts/show/4005
	//#warning complete with http://www.gizma.com/easing/#quint3
	//#warning complete with http://giancarlo.dimassa.net/2007/07/11/elastic-and-bouncing-transitions-in-scriptaculous/
	//#warning complete with http://www.reflektions.com/miniml/template_archives_cat.asp?cat=17

   History:

   v1.0 - (13-Oct-08) 	Initial version


   Visualizations:

	//                                   linear_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - *******+  -
	//    |                                                      *******       |
	//    |                                               ***+***              |
	//    |                                       ********   |                 |
	//    |                                *+*****           |                 |
	//    |                         ******* |                |                 |
	//    |                  *******        |                |                 |
	//d   |          ******+*               |                |                 |
	//e   |   *******      |0.25            |0.50            |0.75             |1.00
	//l 0-+***-------------+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                   quadIn_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  ***+  -
	//    |                                                             ****   |
	//    |                                                         ****       |
	//    |                                                    *****           |
	//    |                                               ***+*                |
	//    |                                         ******   |                 |
	//    |                                   ******         |                 |
	//d   |                           ******+*               |                 |
	//e   |               *+**********      |0.50            |0.75             |1.00
	//l 0-+***************-+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                  quadOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - ****+*****************+  -
	//    |                                     *********    |                 |
	//    |                              ***+***             |                 |
	//    |                        ******   |                |                 |
	//    |                  ******         |                |                 |
	//    |              **+*               |                |                 |
	//    |         *****  |                |                |                 |
	//d   |     ****       |                |                |                 |
	//e   | ****           |0.25            |0.50            |0.75             |1.00
	//l 0-+*---------------+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                quadInOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  ***************+  -
	//    |                                              ****+**               |
	//    |                                         *****    |                 |
	//    |                                     ****         |                 |
	//    |                                 +***             |                 |
	//    |                             ****|                |                 |
	//    |                         ****    |                |                 |
	//d   |                   ******        |                |                 |
	//e   |           *****+**25            |0.50            |0.75             |1.00
	//l 0-+***********-----+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                  cubicIn_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -**+  -
	//    |                                                               ***  |
	//    |                                                            ***     |
	//    |                                                         ***        |
	//    |                                                     ****           |
	//    |                                                 *+**               |
	//    |                                            ***** |                 |
	//d   |                                     *******      |                 |
	//e   |                 0.25    ********+***0            |0.75             |1.00
	//l 0-+****************+********--------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                 cubicOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -**************+*****************+  -
	//    |                           ******+**              |                 |
	//    |                      *****      |                |                 |
	//    |                 *****           |                |                 |
	//    |             ***+                |                |                 |
	//    |         ****   |                |                |                 |
	//    |      ***       |                |                |                 |
	//d   |   ***          |                |                |                 |
	//e   | **             |0.25            |0.50            |0.75             |1.00
	//l 0-+*---------------+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                               cubicInOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -**+*****************+  -
	//    |                                           *****  |                 |
	//    |                                       ****       |                 |
	//    |                                    ***           |                 |
	//    |                                 +**              |                 |
	//    |                               **|                |                 |
	//    |                           ****  |                |                 |
	//d   |                       ****      |                |                 |
	//e   |                +******          |0.50            |0.75             |1.00
	//l 0-+****************+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                  quartIn_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - *+  -
	//    |                                                                 ** |
	//    |                                                              ***   |
	//    |                                                            **      |
	//    |                                                         ***        |
	//    |                                                     ****           |
	//    |                                                 *+**               |
	//d   |                                           ****** |                 |
	//e   |                 0.25            +*********       |0.75             |1.00
	//l 0-+****************+****************+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                 quartOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  ****+****************+*****************+  -
	//    |                      *******    |                |                 |
	//    |                 *****           |                |                 |
	//    |             ***+                |                |                 |
	//    |          ***   |                |                |                 |
	//    |       ***      |                |                |                 |
	//    |     **         |                |                |                 |
	//d   |  ***           |                |                |                 |
	//e   |**              |0.25            |0.50            |0.75             |1.00
	//l 0-+----------------+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                               quartInOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -*****+*****************+  -
	//    |                                         ****     |                 |
	//    |                                      ***         |                 |
	//    |                                   ***            |                 |
	//    |                                  *               |                 |
	//    |                               **+                |                 |
	//    |                             **  |                |                 |
	//d   |                         ****    |                |                 |
	//e   |                 0.******        |0.50            |0.75             |1.00
	//l 0-+****************+**--------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                  quintIn_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - *+  -
	//    |                                                                 ** |
	//    |                                                               **   |
	//    |                                                             **     |
	//    |                                                           **       |
	//    |                                                        ***         |
	//    |                                                    ****            |
	//d   |                                               ***+*                |
	//e   |                 0.25             0.50*********   |0.75             |1.00
	//l 0-+****************+****************+****------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                 quintOut_01
	// +1 |  -  -  -  -  -  -  -  - ********+****************+*****************+  -
	//    |                  *******        |                |                 |
	//    |              **+*               |                |                 |
	//    |          ****  |                |                |                 |
	//    |        **      |                |                |                 |
	//    |      **        |                |                |                 |
	//    |    **          |                |                |                 |
	//d   |  **            |                |                |                 |
	//e   |**              |0.25            |0.50            |0.75             |1.00
	//l 0-+----------------+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                               quintInOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  - *******+*****************+  -
	//    |                                       ****       |                 |
	//    |                                     **           |                 |
	//    |                                   **             |                 |
	//    |                                  *               |                 |
	//    |                                *+                |                 |
	//    |                              ** |                |                 |
	//d   |                           ***   |                |                 |
	//e   |                 0.25******      |0.50            |0.75             |1.00
	//l 0-+****************+****------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                   expoIn_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - *+  -
	//    |                                                                  * |
	//    |                                                                 *  |
	//    |                                                               **   |
	//    |                                                             **     |
	//    |                                                           **       |
	//    |                                                       ****         |
	//d   |                                                  +****             |
	//e   |                 0.25             0.50 ***********|0.75             |1.00
	//l 0-+****************+****************+*****-----------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                  expoOut_01
	// +1 |  -  -  -  -  -  -  - ***********+****************+*****************+  -
	//    |               *+*****           |                |                 |
	//    |           **** |                |                |                 |
	//    |        ***     |                |                |                 |
	//    |      **        |                |                |                 |
	//    |    **          |                |                |                 |
	//    |   *            |                |                |                 |
	//d   | **             |                |                |                 |
	//e   |*               |0.25            |0.50            |0.75             |1.00
	//l 0-+----------------+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                expoInOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -********+*****************+  -
	//    |                                      ****        |                 |
	//    |                                    **            |                 |
	//    |                                   *              |                 |
	//    |                                  *               |                 |
	//    |                                *+                |                 |
	//    |                               * |                |                 |
	//d   |                            ***  |                |                 |
	//e   |                 0.25  *****     |0.50            |0.75             |1.00
	//l 0-+****************+******----------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                   sineIn_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - ****+  -
	//    |                                                           *****    |
	//    |                                                      *****         |
	//    |                                                 *+***              |
	//    |                                            ***** |                 |
	//    |                                      ******      |                 |
	//    |                                *+****            |                 |
	//d   |                         ******* |                |                 |
	//e   |              **+********        |0.50            |0.75             |1.00
	//l 0-+**************--+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                  sineOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -**+*****************+  -
	//    |                                       *********  |                 |
	//    |                                 +*****           |                 |
	//    |                           ******|                |                 |
	//    |                     ******      |                |                 |
	//    |                +****            |                |                 |
	//    |           *****|                |                |                 |
	//d   |      *****     |                |                |                 |
	//e   |  ****          |0.25            |0.50            |0.75             |1.00
	//l 0-+**--------------+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                sineInOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -**************+  -
	//    |                                                **+***              |
	//    |                                          ******  |                 |
	//    |                                     *****        |                 |
	//    |                                 +***             |                 |
	//    |                            *****|                |                 |
	//    |                       *****     |                |                 |
	//d   |                 ******          |                |                 |
	//e   |          ******+0.25            |0.50            |0.75             |1.00
	//l 0-+**********------+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                   circIn_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  +  -
	//    |                                                                   *|
	//    |                                                                 ** |
	//    |                                                              ***   |
	//    |                                                          ****      |
	//    |                                                     *****          |
	//    |                                              ****+**               |
	//d   |                                     *********    |                 |
	//e   |                 0.25 ***********+***0            |0.75             |1.00
	//l 0-+****************+*****-----------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                  circOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  ************+*****************+  -
	//    |                          *******+****            |                 |
	//    |                  ********       |                |                 |
	//    |            ****+*               |                |                 |
	//    |        ****    |                |                |                 |
	//    |    ****        |                |                |                 |
	//    |  **            |                |                |                 |
	//d   |**              |                |                |                 |
	//e   |                |0.25            |0.50            |0.75             |1.00
	//l 0-+----------------+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                circInOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  ***+*****************+  -
	//    |                                        *******   |                 |
	//    |                                    ****          |                 |
	//    |                                  **              |                 |
	//    |                                                  |                 |
	//    |                                 +                |                 |
	//    |                              ***|                |                 |
	//d   |                         *****   |                |                 |
	//e   |               *+********        |0.50            |0.75             |1.00
	//l 0-+***************-+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                   backIn_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - *+  -
	//    |                                                                 ** |
	//    |                                                               **   |
	//    |                                                             **     |
	//    |                                                           **       |
	//    |                                                         **         |
	//    |                                                      ***           |
	//d   |                                                  +***              |
	//e   |                 0.25             0.50        ****|0.75             |1.00
	//l 0-+**************--+----------------+-----*******----+-----------------+----
	//t   |              **+****************+*****            3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                  backOut_01
	// +1 |  -  -  -  -  -  -  *************+****************+*****************+  -
	//    |                +***             |                |                 |
	//    |             ***|                |                |                 |
	//    |          ***   |                |                |                 |
	//    |        **      |                |                |                 |
	//    |      **        |                |                |                 |
	//    |    **          |                |                |                 |
	//d   |  **            |                |                |                 |
	//e   |**              |0.25            |0.50            |0.75             |1.00
	//l 0-+----------------+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                backInOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  *********+*****************+  -
	//    |                                      ***         |                 |
	//    |                                     *            |                 |
	//    |                                   **             |                 |
	//    |                                  *               |                 |
	//    |                                *+                |                 |
	//    |                               * |                |                 |
	//d   |                             **  |                |                 |
	//e   |                 0.25     ***    |0.50            |0.75             |1.00
	//l 0-+********--------+-----****-------+----------------+-----------------+----
	//t   |        ********+*****            PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                 bounceIn_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -********+  -
	//    |                                                         ***        |
	//    |                                                      ***           |
	//    |                                                    **              |
	//    |                                                  +*                |
	//    |                                                 *|                 |
	//    |                                               ** |                 |
	//d   |                       **********+****       **   |                 |
	//e   |          *****  0.2***          |0.50****  *     |0.75             |1.00
	//l 0-+**********-----*+***-------------+--------**------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                bounceOut_01
	// +1 |  -  -  -  -  -  -  -  ***** -  -  -  -  -  -  ***+*****************+  -
	//    |                      *     ****          *****   |                 |
	//    |                    **          *+********        |                 |
	//    |                   *             |                |                 |
	//    |                 **              |                |                 |
	//    |               *+                |                |                 |
	//    |            *** |                |                |                 |
	//d   |         ***    |                |                |                 |
	//e   |     ****       |0.25            |0.50            |0.75             |1.00
	//l 0-+*****-----------+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                              bounceInOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -*****-  - *************+  -
	//    |                                            *     +****             |
	//    |                                          **      |                 |
	//    |                                       ***        |                 |
	//    |                               **+*****           |                 |
	//    |                            ***  |                |                 |
	//    |                         ***     |                |                 |
	//d   |                        *        |                |                 |
	//e   |          ******+***5 **         |0.50            |0.75             |1.00
	//l 0-+**********------+---**-----------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                elasticIn_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  +  -
	//    |                                                                   *|
	//    |                                                                    |
	//    |                                                                  * |
	//    |                                                                    |
	//    |                                                                 *  |
	//    |                                                                    |
	//d   |                                                                *   |
	//e   |                 0.25             0.50       *****+*.75             |1.00
	//l 0-+****************+****************+***********-----+-**---------*----+----
	//t   |                 PI/2             PI               3PI*4      *      2PI
	//a   |                                                       **    *
	//    |                                                         ** *
	//    |                                                           *
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                               elasticOut_01
	// +1 |****************+****************+****************+*****************+  -
	//    |                |                |                |                 |
	//    |                |                |                |                 |
	//    |                |                |                |                 |
	//    |                |                |                |                 |
	//    |                |                |                |                 |
	//    |                |                |                |                 |
	//d   |                |                |                |                 |
	//e   |                |0.25            |0.50            |0.75             |1.00
	//l 0-+----------------+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                             elasticInOut_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  - *************+*****************+  -
	//    |                                    *             |                 |
	//    |                                   *              |                 |
	//    |                                                  |                 |
	//    |                                  *               |                 |
	//    |                                 +                |                 |
	//    |                                *|                |                 |
	//d   |                               * |                |                 |
	//e   |                 0.25            |0.50            |0.75             |1.00
	//l 0-+****************+*******-----**--+----------------+-----------------+----
	//t   |                 PI/2   *****     PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                   sinPi2_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -**+*****************+  -
	//    |                                       *********  |                 |
	//    |                                 +*****           |                 |
	//    |                           ******|                |                 |
	//    |                     ******      |                |                 |
	//    |                +****            |                |                 |
	//    |           *****|                |                |                 |
	//d   |      *****     |                |                |                 |
	//e   |  ****          |0.25            |0.50            |0.75             |1.00
	//l 0-+**--------------+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                acelBreak_01
	// +1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -**************+  -
	//    |                                                **+***              |
	//    |                                          ******  |                 |
	//    |                                     *****        |                 |
	//    |                                 +***             |                 |
	//    |                            *****|                |                 |
	//    |                       *****     |                |                 |
	//d   |                 ******          |                |                 |
	//e   |          ******+0.25            |0.50            |0.75             |1.00
	//l 0-+**********------+----------------+----------------+-----------------+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                   cos2Pi_11
	// +1 +*****-  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -*****+  -
	//    |     **                                                      **     |
	//    |       *                                                    *       |
	//    |        **                                                **        |
	//    |          *                                              *          |
	//    |           **                                          **           |
	//    |             *                                        *             |
	//d   |              *                                      *              |
	//e   |               * 0.25             0.50             0*75             |1.00
	//l 0-+----------------+----------------+----------------+*----------------+----
	//t   |                 *I/2            |PI              +3PI/4             2PI
	//a   |                  **             |              **
	//    |                    *            |             *
	//    |                     *           |            *
	//    |                      **         |          **
	//    |                        *        |         *
	//    |                         **      |       **
	//    |                           **    |     **
	// -1 |  -  -  -  -  -  -  -  -  -  ****+*****  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                   sin2Pi_00
	// +1 |  -  -  -  -****+***** -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |         ***    |     **
	//    |        *       |       **
	//    |      **        |         *
	//    |     *          |          **
	//    |    *           |            *
	//    |  **            |             *
	//d   | *              |              *
	//e   |*               |0.25           * 0.50             0.75              1.00
	//l 0-+----------------+----------------+*---------------+-----------------+----
	//t   |                 PI/2             P*              |3PI/4           * 2PI
	//a   |                                    *             |               *
	//    |                                     *            |             **
	//    |                                      *           |            *
	//    |                                       **         |           *
	//    |                                         *        |         **
	//    |                                          **      |        *
	//    |                                            **    |     ***
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - ****+*****-  -  -  -  -  -
	//    |                                time

	//                                    sinPi_00
	// +1 |  -  -  -  -  -  -  -  -*********+**********-  -  -  -  -  -  -  -  -  -
	//    |                   *****         |          *****
	//    |                +**              |               *+*
	//    |             ***|                |                | ***
	//    |          ***   |                |                |    ***
	//    |        **      |                |                |       **
	//    |     ***        |                |                |         ***
	//d   |   **           |                |                |            **
	//e   | **             |0.25            |0.50            |0.75          **  1.00
	//l 0-+*---------------+----------------+----------------+----------------*+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                 sinPi2Pi_10
	// +1 +****************+***-  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                |   *********
	//    |                |            ****+*
	//    |                |                | ******
	//    |                |                |       ******
	//    |                |                |             ***+*
	//    |                |                |                | *****
	//d   |                |                |                |      *****
	//e   |                |0.25            |0.50            |0.75       ****   1.00
	//l 0-+----------------+----------------+----------------+---------------**+----
	//t   |                 PI/2             PI               3PI/4             2PI
	//a   |
	//    |
	//    |
	//    |
	//    |
	//    |
	//    |
	// -1 |  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time

	//                                   sin4Pi_00
	// +1 |  -  -*****-  -  -  -  -  -  -  -  -  - *****  -  -  -  -  -  -  -  -  -
	//    |    **     *                           *     *
	//    |            *                         *       *
	//    |   *                                 *         *
	//    |  *          *                                  *
	//    |              *                     *
	//    | *                                 *             *
	//d   |*              *
	//e   |                 0.25             *.50            +0.75              1.00
	//l 0-+----------------+----------------+----------------+*----------------+----
	//t   |                 *I/2            +PI               3PI/4             2PI
	//a   |                                                    *              *
	//    |                  *             *                                 *
	//    |                               *                     *
	//    |                   *                                  *          *
	//    |                    *         *                                 *
	//    |                     *       *                         *
	//    |                      *     *                           *     **
	// -1 |  -  -  -  -  -  -  -  ***** -  -  -  -  -  -  -  -  -  -*****-  -  -  -
	//    |                                time

	//                                  sin3Pi4_00
	// +1 |  -  -  ******-  -  -  -  -  -  -  -  -  -  -  -  -  -******  -  -  -  -
	//    |      **      **                                    **      **
	//    |     *          +                                  *          *
	//    |    *           |*                                +            *
	//    |   *            | *                              *|             *
	//    |  *             |  *                            * |              *
	//    | *              |   *                          *  |               *
	//d   |                |                                 |
	//e   |*               |0.25*            0.50        *   |0.75            * 1.00
	//l 0-+----------------+-----*----------+-----------*----+-----------------+----
	//t   |                 PI/2  *         |PI        *      3PI/4             2PI
	//a   |                                 |
	//    |                        *        |         *
	//    |                         *       |        *
	//    |                          *      |       *
	//    |                           *     |      *
	//    |                            *    |     *
	//    |                             **  |   **
	// -1 |  -  -  -  -  -  -  -  -  -  - **+*** -  -  -  -  -  -  -  -  -  -  -  -
	//    |                                time
*/

#ifndef __EASE_H__
#define __EASE_H__

#include <math.h>

namespace rag
{

/// Collection of code-generated curves useful to create procedural tween animations.
class Ease
{

public:

	/// The types of curve supported.
	enum EaseType
	{
		linear_01,

		quadIn_01,
		quadOut_01,
		quadInOut_01,
		cubicIn_01,
		cubicOut_01,
		cubicInOut_01,
		quartIn_01,
		quartOut_01,
		quartInOut_01,
		quintIn_01,
		quintOut_01,
		quintInOut_01,
		expoIn_01,
		expoOut_01,
		expoInOut_01,
		sineIn_01,
		sineOut_01,
		sineInOut_01,
		circIn_01,
		circOut_01,
		circInOut_01,
		backIn_01,
		backOut_01,
		backInOut_01,
		bounceIn_01,
		bounceOut_01,
		bounceInOut_01,
		elasticIn_01,
		elasticOut_01,
		elasticInOut_01,

		sinPi2_01,
		acelBreak_01,
		cos2Pi_11,

		sin2Pi_00,
		sinPi_00,
		sinPi2Pi_10,
		sin4Pi_00,
		sin3Pi4_00
	};

    EaseType type;

    float PIf;

	/// Constructs an Ease curve.
    Ease(EaseType myType = linear_01):
		PIf(3.1415926535897932384626433832795f)
    {
      type = myType;
    }

	/// Returns the current type.
    EaseType getType()
    {
      return type;
    }

	/// Sets the current EaseType.
    void setType(EaseType myType)
    {
      type = myType;
    }

	/// Returns the y coordinate of the curve for a given point t in a curve of length d.
    float get(float t /*current (t)*/, float d /*total (t)*/)
    {
        
        if(t >= d)
        {
            if(type >= sin2Pi_00)
                return 0;
            
            return 1;
        }

      switch(type)
      {
        case linear_01:
        {
          return t/d;
        }

        case sin2Pi_00:
        {
          float delta = t/d;
          return sin(delta * 2.0f * PIf);
        }

        case sinPi_00:
        {
          float delta = t/d;
          return sin(delta * PIf);
        }

        case sinPi2Pi_10:
        {
          float delta = t/d;
          return sin((0.5f * PIf) * (delta + 1));
        }

        case sin4Pi_00:
        {
          float delta = t/d;
          return sin(delta * 4.0f * PIf);
        }

        case sin3Pi4_00:
        {
          float delta = t/d;
          return sin(delta * 3.0f * PIf);
        }

        case sinPi2_01:
        {
          float delta = t/d;
          return sin(delta * 0.5f * PIf);
        }

        case acelBreak_01:
        {
          float delta = t/d;
          return (sin((delta * PIf) - (PIf * 0.5f)) + 1.0f) * 0.5f;
        }

        case cos2Pi_11:
        {
          float delta = t/d;
          return cos(delta * 2.0f * PIf);
        }


        case backIn_01: //?
        {
          float s = 1.70158f;
          float postFix = t/=d;

          return (postFix)*t*((s+1)*t - s);
        }

        case backOut_01: //?
        {
          float s = 1.70158f;

          return (float)(1.0*((t=t/d-1)*t*((s+1)*t + s) + 1));
        }

        case backInOut_01: //?
        {
          float s = 1.70158f;

          if ((t/=d/2) < 1)
            return (float)(1.0/2*(t*t*(((s*=(1.525f))+1)*t - s)));

          float postFix = t-=2;

          return (float)(1.0/2*((postFix)*t*(((s*=(1.525f))+1)*t + s) + 2));
        }


        #define bounce \
          if ((t/=d) < (1/2.75f)) \
          { \
            v = (float)(1.0f*(7.5625f*t*t)); \
          } \
          else if (t < (2.0f/2.75f)) \
          { \
            float postFix = t-=(1.5f/2.75f); \
            \
            v = 1.0f*(7.5625f*(postFix)*t + .75f); \
          } \
          else if (t < (2.5f/2.75f)) \
          { \
            float postFix = t-=(2.25f/2.75f); \
          \
            v = 1.0f*(7.5625f*(postFix)*t + .9375f); \
          } \
          else \
          { \
            float postFix = t-=(2.625f/2.75f); \
          \
            v = 1.0f*(7.5625f*(postFix)*t + .984375f); \
          }

        case bounceIn_01: //?
        {
          float v;

          t = d-t;

          bounce

          return 1.0f - v;
        }

        case bounceOut_01: //?
        {
          float v;

          bounce

          return v;
        }

        case bounceInOut_01: //?
        {
          float v;

          if (t < d/2)
          {
            t = t*2;

            t = d-t;

            bounce

            return (float)((1.0 - v) * .5f);
          }
          else
          {
            t = t*2 -d;

            bounce

            return (float)(v * .5f + 1.0*.5f);
          }
        }

        #undef bounce


        case circIn_01:
          t /= d;
          return (float)(1.0 - sqrt(1 - t*t));

        case circOut_01: //*
          t /= d;
          t--;
          return (float)(sqrt(1 - t*t));

        case circInOut_01: //*
          t /= d/2;
          if(t < 1)
            return (float)(-1.0/2 * (sqrt(1 - t*t) - 1));

          t-=2;

          return (float)(1.0/2 * (sqrt(1 - t*t) + 1));


        case elasticIn_01: //?
        {
          //if (t==0)
          //	return 0;

          t/=d;

          float p=d*.3f;
          float a=1.0;
          float s=p/4;
          float postFix =a*(float)(pow(2,10*(t-=1))); // this is a fix, again, with post-increment operators

          return (float)(-(postFix * sin((t*d-s)*(2*PIf)/p )));
        }

        case elasticOut_01: //?
        {
          //if (t==0)
          //	return 0;

          //if ((t/=d)==1)
          //	return 1.0;

          float p=d*.3f;
          float a=1.0;
          float s=p/4;

          return (float)((a*pow(2,-10*t) * sin( (t*d-s)*(2*PIf)/p ) + 1.0));
        }

        case elasticInOut_01: //?
        {
          //if (t==0)
          //	return 0;

          t/=d/2;
          //if ((t/=d/2)==2)
          //	return 1.0;

          float p=d*(.3f*1.5f);
          float a=1.0;
          float s=p/4;

          if (t < 1)
          {
            float postFix = (float)(a*pow(2,10*(t-=1))); // postIncrement is evil

            return (float)(-.5f*(postFix* sin( (t*d-s)*(2*PIf)/p )));
          }

          float postFix =  a*(float)(pow(2,-10*(t-=1))); // postIncrement is evil

          return (float)(postFix * sin( (t*d-s)*(2*PIf)/p )*.5f + 1.0);
        }



        case expoIn_01: //*
          return (float)(pow(2, 10 * (t/d - 1)));

        case expoOut_01: //*
          return (float)(1.0 - ( t == d ? 0 : pow(2.0, -10.0 * (t/d))));

        case expoInOut_01: //*

          //if( t == 0 ) //new
          //	;//return 0.0;

          //if( t == d) //new
          //	return 1.0;

          t /= d/2;

          if (t < 1)
            return (float)(1.0/2 * pow(2, 10 * (t - 1)));

          t--;

          return (float)(1.0/2 * (-pow(2, -10 * t) + 2));



        case quadIn_01: //*
          t /= d;
          return (float)(t*t);

        case quadOut_01: //*
          t /= d;
          return (float)((2.0 - t) * t);

        case quadInOut_01: //*
          t /= d/2;

          if(t < 1)
            return (float)(1.0/2*t*t);

          t--;

          return (float)(-1.0/2 * (t*(t-2) - 1));


        case cubicIn_01: //*
          t /= d;
          return (float)(t*t*t);

        case cubicOut_01: //*
          t /= d;
          t--;
          return (float)(1.0 + t*t*t);

        case cubicInOut_01: //*
          t /= d/2;

          if (t < 1)
            return (float)(1.0/2*t*t*t);

          t -= 2;

          return (float)(1.0/2*(t*t*t + 2));



        case quartIn_01: //*
          t /= d;
          return (float)(t*t*t*t);

        case quartOut_01: //*
          t /= d;
          t--;
          return (float)(1.0 - t*t*t*t);

        case quartInOut_01: //*

          t /= d/2;

          if(t < 1)
            return (float)(1.0/2*t*t*t*t);

          t -= 2;

          return (float)(-1.0/2 * (t*t*t*t - 2));



        case quintIn_01: //*
          t /= d;
          return (float)(t*t*t*t*t);

        case quintOut_01: //*
          t /= d;
          t--;
          return (float)(1.0 + t*t*t*t*t);

        case quintInOut_01: //*

          t /= d/2;

          if(t < 1)
            return (float)(1.0/2*t*t*t*t*t);

          t -= 2;

          return (float)(1.0/2*(t*t*t*t*t + 2));



        case sineIn_01: //*
          return (float)(1.0 - cos(t/d * (PIf/2)));

        case sineOut_01: //*
          return sin(t/d * (PIf/2));

        case sineInOut_01: //*
          return (float)(-1.0/2 * (cos(PIf*t/d) - 1));

        default:
          //undefined();
          return 0;
      }
    }
};

} // rag

#endif
