#ifndef Rag_Event_h
#define Rag_Event_h

#include <string>

namespace rag
{
    class DisplayObject;
}

namespace events
{
    
    static const std::string REMOVED = "Removed";
	static const std::string ADDED = "Added";

	/// Base class for event system.
	/** When an event is triggered, it is processed this way:
	* The display list is travelled from leafs to stage (AKA root).
	* The propagation stops when the root is reached or when a node with
	* the property 'captureInput' set to true is traversed.
	* \sa EventListener, EventDispatcher
	*/
    class Event
    {
        
    public:
        
		/// Creates a new event of the given type.
        Event(std::string type)
        {
            this->type = type;
            captured = false;
        }
        
        std::string             type;		///< The type of the event. The string should be unique for this event.
        bool                    captured;	///< When an event is captured, it won't propagate anymore through the Display List.
        rag::DisplayObject*     target;		///< Usually points to the dispatcher object. 'target' can be assigned to anything for custom events.
        
		/// string representing the event.
        virtual std::string toString() {return "Event " + type;}
        
    };
}


#endif
