#ifndef _IMAGE_H_
#define _IMAGE_H_

#include <rag/Platform.h>
#include <rag/ResourceMgr.h>
#include <rag/Color.h>
#include <string>
#include <vector>

namespace rag
{
    class ImageLoaderJPG;
	class ImageLoader;
    
	/// Image object.
	/** An image represents a 2D texture, usually readed from a specific file on disk.
		Images can be drawn by Bitmap instances. Image raw data can be read and/or manipulated.
	*/
    class Image: public Resource
    {
    public:

		/// Returns a functional image with size, the image is loaded in background.
		/** Asks resource manager for the image, create it if not exists.
		*/
		static Image* loadImage(const std::string& path = "", int textureWrapMode = GL_CLAMP_TO_EDGE, bool deleteImageData = true, bool downloaded = false);
        
		/// Returns a functional image with size, the image is loaded in background.
		/** If you want direct access to image in raw format, you need to specify deleteImageData = false, otherwise image data is deleted.
		*/
        Image(const std::string& path = "", int textureWrapMode = GL_CLAMP_TO_EDGE, bool deleteImageData = true, bool downloaded = false);

		/// Default destructor.
		virtual ~Image();
		
		/// CPU intensive load goes here.
		virtual void loadInBackground() override;

		/// The part of the loading that must be done in main thread.
		virtual void loadSync() override;

		/// on context lost, images can be reloaded.
		void reload();

		/// Adds a compressed folder.
		/** All pngs and jpgs loaded inside a compressed folder
			are converted to 16bpp images in memory.
		*/
		static void setCompressedFolder(std::string folder);

		/// Clears all compressed folders.
		static void clearCompressedFolders();
        
		int width, height;
		int pixelFormat;
        GLuint name;        
		GLubyte *bytes;

		static int s_memorySize;
        
    private:
        
		friend class ImageLoaderJPG; // TODO Refactor downloaded stuff.
		
		void init();

        int nextpot(unsigned int n);
		void* rgba8888_to_rgba4444(void* src);

		int textureWrapMode;
        
		bool deleteImageData;
		bool isDownloaded;

		static std::vector<std::string> compressedFolders;

		ImageLoader* imageLoader;
    };
}

#endif