#include "ImageLoaderJPG.h"

using namespace rag;

ImageLoaderJPG::ImageLoaderJPG(const std::string& name, rag::Image* image) :
name(name),
image(image),
byte(0),
buffer(NULL)
{
	info.err = jpeg_std_error(&err);
	jpeg_create_decompress(&info);
}

ImageLoaderJPG::~ImageLoaderJPG()
{
	jpeg_destroy_decompress(&info);

	free(buffer);
	buffer = NULL;
}


bool ImageLoaderJPG::loadInfo()
{
	bool isBundle = !image->isDownloaded;
	rag::File file(name, isBundle);
	if (!file.open("rb", false))
	{
		buffer = NULL;
		byte = 0;
		return false;
	}

	filesize = file.getSize();
	buffer = (unsigned char*)malloc(filesize);
	file.read(buffer, filesize);
	file.close();

	jpeg_mem_src(&info, buffer, filesize);

	int rc = jpeg_read_header(&info, TRUE);
	if (rc != 1) {
		free(buffer);
		buffer = NULL;
		return false;
	}

	jpeg_start_decompress(&info);

	int width = info.output_width;
	if ((width % 4) != 0) //adjust size to a multiple of 4
		width = (width + 4 - (width % 4));

	image->width = width;
	image->height = info.output_height;
	int pixel_size = info.output_components;
	
	switch (pixel_size)
	{
	case 1: image->pixelFormat = GL_ALPHA; break;
	case 3: image->pixelFormat = GL_RGB; break;
	default:
		image->pixelFormat = GL_RGBA;
		break;
	}

	return true;
}

bool ImageLoaderJPG::loadImage()
{
	if (!buffer)
		return false;

	int pixel_size = info.output_components;
	int width = info.output_width;
	int height = info.output_height;

	if ((width % 4) != 0) //adjust size to a multiple of 4
		width = (width + 4 - (width % 4));

	int bmp_size = width * height * pixel_size;

	image->bytes =(GLubyte*) malloc(bmp_size);

	int row_stride = width * pixel_size;

	while (info.output_scanline < info.output_height) {
		unsigned char *buffer_array[1];
		buffer_array[0] = image->bytes + (info.output_scanline) * row_stride;
 
		jpeg_read_scanlines(&info, buffer_array, 1);
 	}

	jpeg_finish_decompress(&info);

    return true;
}