#include "ImageLoaderPNG.h"
#include <rag/Platform.h>
#include <rag/File.h>
#include <rag/Color.h>

using namespace rag;

void PNGAPI user_read_data_fcn(png_structp png_ptr, png_bytep data, png_size_t length)
{
	ImageLoaderPNG* in = (ImageLoaderPNG*)png_ptr->io_ptr;
	memcpy(data, &in->buffer[in->byte], length);

	in->byte += length;
}

ImageLoaderPNG::ImageLoaderPNG(const std::string& name, rag::Image* image):
	name(name),
	image(image),
	png_ptr(NULL),
	info_ptr(NULL),
	byte(0),
	buffer(NULL)
{
}

ImageLoaderPNG::~ImageLoaderPNG()
{
	// clean up
	if (png_ptr)
	{
		png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
	}
}


bool ImageLoaderPNG::loadInfo()
{
	rag::File file(name);
	if (!file.open())
	{
		buffer = NULL;
		return false;
	}

	filesize = file.getSize();
	buffer = (unsigned char*)malloc(filesize);
	file.read(buffer, filesize);
	file.close();

	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
		NULL, NULL, NULL);

	if (png_ptr == NULL) {
		return false;
	}

	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL) {
		png_destroy_read_struct(&png_ptr, png_infopp_NULL, png_infopp_NULL);
		return false;
	}

	if (setjmp(png_jmpbuf(png_ptr))) {
		png_destroy_read_struct(&png_ptr, &info_ptr, png_infopp_NULL);
		return false;
	}

	png_set_read_fn(png_ptr, this, user_read_data_fcn); // Set our read function
	
	int png_transforms = 0;
	png_set_palette_to_rgb(png_ptr);
	png_read_info(png_ptr, info_ptr);

	image->width = png_get_image_width(png_ptr, info_ptr);
    image->height = png_get_image_height(png_ptr, info_ptr);
	int color_type = png_get_color_type(png_ptr, info_ptr);
    int bit_depth = png_get_bit_depth(png_ptr, info_ptr);

	image->pixelFormat = GL_RGBA;
	switch (color_type)
	{
		case PNG_COLOR_TYPE_RGBA: image->pixelFormat = GL_RGBA; break;
		case PNG_COLOR_TYPE_RGB: image->pixelFormat = GL_RGB; break;
		case PNG_COLOR_TYPE_GRAY: image->pixelFormat = GL_ALPHA; break;
		case PNG_COLOR_TYPE_PALETTE:
			{
				image->pixelFormat = info_ptr->num_trans == 0 ? GL_RGB : GL_RGBA;
			}; break;
	}

	return true;
}

bool ImageLoaderPNG::loadImage() {

	if (!png_ptr)
		return false;

	// Create one memory block for openGL
	png_read_update_info(png_ptr, info_ptr);

	// alloc memory for row pointers
	png_bytepp row_pointers;
	row_pointers = (png_bytep*) malloc(sizeof(png_bytep) * image->height);

	// find out rowbytes.
	int channels = 0;
	switch (image->pixelFormat)
	{
		case GL_RGBA: channels = 4; break;
		case GL_RGB: channels = 3; break;
		case GL_ALPHA: channels = 1; break;
	}
	int rowbytes = image->width * channels;
	image->bytes = (GLubyte*)malloc(rowbytes * image->height);

	// make row pointer point to correct addresses
    for (int y=0; y<image->height; y++)
	{
		row_pointers[y] = &image->bytes[rowbytes * y];
	}

	png_start_read_image(png_ptr);
	png_read_image(png_ptr, row_pointers);

	// Once image is read we can free file raw data buffer.
	free(buffer);
	buffer = NULL;
	free(row_pointers);

    return true;
}