#ifndef _InputManager_h
#define _InputManager_h

#include <rag/TouchEvent.h>
#include <rag/DisplayObject.h>
#include <vector>
#include <map>

namespace rag
{

/// Simple Input Manager
class InputManager {

public:

	/// Returns the shared instance of the InputManager.
	static InputManager&   getInstance();

	/// Introduces an input event in the system.
	void processInputEvent(events::TouchEvent& e);

	/// Prevents input to be processed.
	void lock();

	/// After lock(), returns to normal operation.
	void unlock();

	/// Returns true if input is currently locked. \see lock(), unlock()
	bool isLocked();

	/// Traverses Display List recursively.
	void traverse(rag::DisplayObject* root);

private:

	InputManager();
	~InputManager();

	void touchEventTraversal(rag::DisplayObject* displayObject, events::TouchEvent& event);

	bool locked;

	std::vector<events::TouchEvent> eventsList;

	struct Touch {

		Touch() :active(false) {};

		glm::vec2   pos;
		glm::vec2   disp;
		bool    active;
		Timer   tapTimer;
	};

	Touch touches[10];

};

} // rag
#endif
