#include "Keyboard.h"
#include "KeyboardManager.h"
#include <rag/KeyboardEvent.h>
#include <rag/Platform.h>
#include "external/NDKHelper/NDKHelper.h"

using namespace rag;
using namespace events;

rag::Rectangle Keyboard::size;

Keyboard::Keyboard(KeyboardType type):
type(type),
shown(false)
{
}

void Keyboard::show()
{
    if (shown) return;
    
#ifdef _WIN32
	WinUtils::getInstance().addEventListener(events::WIN_EVENT, this);
#elif __APPLE__
    iosUtils::setKeyboardType(type);
    iosUtils::showKeyboard();
#elif __ANDROID__
    ndkhelper.sendMessage("showKeyboard");
    ndkhelper.addEventListener("KeyPress", this);
    ndkhelper.addEventListener("DeleteKeyPress", this);
    ndkhelper.addEventListener("KeyboardSizeRetrieved", this);
#endif
    
    KeyboardManager::getInstance().addEventListener(KEY_UP, this);
    
    shown = true;
}

void Keyboard::hide()
{
    if (!shown) return;
    
    KeyboardManager::getInstance().removeEventListener(KEY_UP, this);
    
#ifdef _WIN32
	WinUtils::getInstance().removeEventListener(events::WIN_EVENT, this);
#elif __APPLE__
    rag::iosUtils::hideKeyboard();
#elif __ANDROID__
    ndkhelper.sendMessage("hideKeyboard");
    ndkhelper.removeEventListener("KeyPress", this);
    ndkhelper.removeEventListener("DeleteKeyPress", this);
    ndkhelper.removeEventListener("KeyboardSizeRetrieved", this);
#endif
    
    shown = false;
}

void Keyboard::onEvent(const std::string& type, events::Event& event)
{
	// Android
    if (type == "KeyPress")
    {
        events::NDKMessage& ndkEvent = static_cast<events::NDKMessage&>(event);
        
        if (!ndkEvent.json) return;
        
        json_t *object = json_object_get(ndkEvent.json, "value");
        if (!object) return;
        
        KeyboardManager::getInstance().nativeInsertText(json_string_value(object));
    }
    else if (type == "DeleteKeyPress")
    {
        KeyboardManager::getInstance().nativeDeleteBackward();
    }
    else if (type == "KeyboardSizeRetrieved")
    {
        events::NDKMessage& ndkEvent = static_cast<events::NDKMessage&>(event);
        Keyboard::size.height = json_integer_value(json_object_get(ndkEvent.json, "height"));
    }

	// Windows
	if (type == events::WIN_EVENT)
	{
		auto e = static_cast<WinEvent&>(event);
		KeyboardManager::getInstance().nativeInsertText(e.toString()); // TODO right implementation.
	}
    
    dispatchEvent(event);
}
