#ifndef Rag_Keyboard_h
#define Rag_Keyboard_h

#include <rag/DisplayObject.h>

namespace rag
{

/// Multiplatform keyboard abstraction.
/** Once created, a Keyboard instance will open a native keyboard
	in the device and will dispatch keyboard events.
	\see events::KeyboardEvent.
*/
class Keyboard : public rag::DisplayObject, events::EventListener
{

public:

	enum KeyboardType
	{
		KeyboardTypeDefault = 0,
		KeyboardTypeEmail
	};

	/// Default constructor.
	/** Creates the keyboard instance.  You need to call show() to see the native keyboard.
		\see show()
	*/
	Keyboard(KeyboardType type = KeyboardTypeDefault);

	/// Default destructor.
	virtual ~Keyboard(){};

	/// Shows the native keyboard.
	void show();

	/// Hides the native keyboard.
	void hide();

	/// Keyboard implements EventListener. Here is where is listening native events.
	virtual void onEvent(const std::string& type, events::Event& event);

	/// The size of the native keyboard rectangle.
	static rag::Rectangle size;

private:

	bool shown;
	KeyboardType type;

};

} // rag

#endif
