#include "Material.h"
#include <glm/gtc/type_ptr.hpp>

using namespace rag;

Material::Material(const std::string& vsh, const std::string& fsh):
blendMode(Material::BLEND_NORMAL),
textureId(0),
priority(0),
stopsPropagation(false)
{
	this->vsh = vsh;
	this->fsh = fsh;
}

void Material::setUniform(const std::string &name, float value)
{
    TUniformVar* uf = getUniform(name);
    if (uf)
        uf->value[0] = value;
    else
    {
        TUniformVar newuf;
            newuf.name =  name;
            newuf.value[0] = value;
            newuf.type = UNIF_1F;
        uniforms.push_back(newuf);   
    }
}

void Material::setUniform(const std::string &name, const glm::vec2& value)
{
	TUniformVar* uf = getUniform(name);
	if (uf)
	{
		uf->value[0] = value.x;
		uf->value[1] = value.y;
	}
	else
	{
		TUniformVar newuf;
		newuf.name = name;
		newuf.value[0] = value.x;
		newuf.value[1] = value.y;
		newuf.type = UNIF_2F;
		uniforms.push_back(newuf);
	}
}

void Material::setUniform(const std::string &name, const glm::vec3& value)
{
    TUniformVar* uf = getUniform(name);
    if (uf)
    {
        uf->value[0] = value.x;
        uf->value[1] = value.y;
        uf->value[2] = value.z;
    }
    else
    {
        TUniformVar newuf;
            newuf.name =  name;
            newuf.value[0] = value.x;
            newuf.value[1] = value.y;
            newuf.value[2] = value.z;
            newuf.type = UNIF_3F;
        uniforms.push_back(newuf);   
    }
}

void Material::setUniform(const std::string &name, const glm::vec4& value)
{
    TUniformVar* uf = getUniform(name);
    if (uf)
    {
        uf->value[0] = value.x;
        uf->value[1] = value.y;
        uf->value[2] = value.z;
		uf->value[3] = value.w;
    }
    else
    {
        TUniformVar newuf;
            newuf.name =  name;
            newuf.value[0] = value.x;
            newuf.value[1] = value.y;
            newuf.value[2] = value.z;
			newuf.value[3] = value.w;
            newuf.type = UNIF_4F;
        uniforms.push_back(newuf);   
    }
}
void Material::setUniform(const std::string &name, const glm::mat4& value)
{	
    TUniformVar* uf = getUniform(name);
    if (uf)
    {
        memcpy( uf->value, glm::value_ptr(value) , 16*sizeof(float) );
    }
    else
    {
        TUniformVar newuf;
            newuf.name =  name;
            memcpy( newuf.value , glm::value_ptr(value) , 16*sizeof(float) );
            newuf.type = UNIF_MAT4;
        uniforms.push_back(newuf);   
    }
}

void Material::setUniform(const std::string &name, const glm::mat3& value)
{	
    TUniformVar* uf = getUniform(name);
    if (uf)
    {
        memcpy( uf->value, glm::value_ptr(value) , 9*sizeof(float) );
    }
    else
    {
        TUniformVar newuf;
            newuf.name =  name;
            memcpy( newuf.value , glm::value_ptr(value) , 9*sizeof(float) );
            newuf.type = UNIF_MAT3;
        uniforms.push_back(newuf);   
    }
}

TUniformVar* Material::getUniform(const std::string &name)
{
    for(std::vector<TUniformVar>::iterator it = uniforms.begin(); it != uniforms.end(); ++it)
        if (it->name == name)
            return &(*it);    
    
    return NULL;
}