//
//  DisplayMovieClip.h
//  
//
//  Created by Iván Sánchez on 11/30/11.
//

#ifndef rag_DisplayMovieClip_h
#define rag_DisplayMovieClip_h

#include "DisplayObject.h"

namespace rag
{
	/// Allows to use imported animations created by Flash CS tool.
    class MovieClip: public DisplayObject
    {
            
    public:
            
		/// Internal of MovieClip, represents a single frame.
        struct Frame
        {
            int index;
            int duration;
            std::string label;
            DisplayObject* bitmap;
        };
            
        MovieClip(std::vector<Frame> frames);
        virtual ~MovieClip();
            
        /// Specifies the number of the frame in which the playhead is located in the timeline of the MovieClip instance.
        int getCurrentFrame (  );
            
        /// Returns the label in the current frame. It may be empty.
        std::string getCurrentFrameLabel (  );
            
		/// Returns the total number of frames in the MovieClip.
        int getTotalFrames (  );
            
        /// Simple playback
        /**
        * Moves the playhead in the timeline of the movie clip. 
        */
        void play (  );
            
        /// Stops the playhead in the movie clip. 
        void stop (  );
            
        /// Goes to a specific frame, then starts playing from there.
        void gotoAndPlay ( int frame );

		/// Goes to a specific frame, then starts playing from there.
        void gotoAndPlay ( const std::string& frame, bool loop, bool forceFirstFrame = false);
            
		/// Goes to a specific frame, then stops there.
        void gotoAndStop ( int frame );

		/// Goes to a specific frame by name, then stops there.
        void gotoAndStop ( const std::string& frame );
            
        /// Sends the playhead to the next frame and stops it.
        void nextFrame (  );
            
        /// Sends the playhead to the previous frame and stops it.
        void prevFrame (  );
            
		/// Returns the current frame internals.
        Frame* getCurrentFrameNode();
            
        /// Display object update
		virtual void logicUpdate() override;

		/// Sets the speed in frames per second the MovieClip should use.
		void setFPS(int fps);
            
		/// Replace an instance of a Displayobject named "name" with a library item called "replacement".
		void replace(const std::string& name, const std::string& library, const std::string& replacement);

		/// Returns true if is currently playing.
		bool isPlaying();

		/// Returns all Frame instances.
		std::vector<Frame>& getFrames();

		/// MovieClip filename.
		std::string fileName;
            
    private:
            
        void setFrame(int frame);
        int nextFrameNum();
            
        std::vector<Frame> frames;
        Frame* currentFrame;
        int currentIndex;
        float elapsedFrameTime;
        bool playing;
        int framesLoaded;

		std::string animLoop; ///< This is not like flash but is handy. We keep looping an animation.
		bool loop;
		int fps;
    };
} // rag

#endif
