#ifndef _RAG_PLATFORM_H_
#define _RAG_PLATFORM_H_




#include <algorithm>




#ifdef _WIN32

#pragma warning( once : 4018 4244 4305 )

#include <Windows.h>
#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>
#include <EGL/egl.h>
#include "win/winUtils.h"

#define HAVE_STRUCT_TIMESPEC // pthreads compatibility in Windows 7

#define  LOGI(...) WinUtils::win32_printf(__VA_ARGS__);
#define  LOGE(...) WinUtils::win32_printfe(__VA_ARGS__);

#endif



#ifdef __APPLE__

#include <OpenGLES/ES2/gl.h>
#include <OpenGLES/ES2/glext.h>
#include "ios/iosUtils.h"
#include <sys/time.h>



#define  LOGI(...)  {rag::iosUtils::iosLog(__VA_ARGS__);}
#define  LOGE(...)  {rag::iosUtils::iosLogError(__VA_ARGS__);}



#endif // __APPLE__



#ifdef ANDROID

#include <jni.h>
#include <android/log.h>
#include <android_native_app_glue.h>
#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>
#include "android/androidUtils.h"
#include <sys/time.h>

#define  STRINGIFY(x) #x
#define  LOG_TAG    __FILE__ ":" STRINGIFY(__LINE__)
#define  LOGI(...) ((void)__android_log_print(ANDROID_LOG_INFO,LOG_TAG, __VA_ARGS__))
#define  LOGE(...) ((void)__android_log_print(ANDROID_LOG_ERROR,LOG_TAG,__VA_ARGS__))

#else

#ifdef __linux__
#include "linux/linuxUtils.h"
#define  LOGI(...)  {linuxUtils::log(__VA_ARGS__);}
#define  LOGE(...)  {linuxUtils::log(__VA_ARGS__);}
#include <stdio.h>
#include <sys/time.h>
#include <string.h>
#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>
#endif

#endif



#endif
