#include "Rectangle.h"
#include <math.h>
#include <algorithm>
#include <sstream>

using namespace rag;

Rectangle::Rectangle(float x, float y, float width, float height)
{
    this->x = x;
    this->y = y;
    this->width = width;
    this->height = height;
}

Rectangle Rectangle::rectUnion(const Rectangle& toUnion) const
{
    if (toUnion.width == 0 && toUnion.height == 0)
        return Rectangle(x, y, width, height);
    
    if (width == 0 && height == 0)
        return toUnion;
    
    float left = std::min(x, toUnion.x);
    float top = std::min(y, toUnion.y);
    float right = std::max(x + width, toUnion.x + toUnion.width);
    float bottom = std::max(y + height, toUnion.y + toUnion.height);
    return Rectangle(left, top, right - left, bottom - top);
}

Rectangle Rectangle::intersection(const rag::Rectangle& toIntersect) const
{
    using namespace std;
    
    float left = max(toIntersect.x, x);
    float top = max(toIntersect.y, y);
    float right = min(toIntersect.width + toIntersect.x, width + x);
    float bottom = min(toIntersect.height + toIntersect.y, height + y);
	if (left < right && top < bottom)
		return Rectangle(left, top, right-left, bottom-top);
    return Rectangle(); // no intersection
}

bool Rectangle::intersects(const rag::Rectangle& toIntersect) const
{
	return (x < toIntersect.x + toIntersect.width && 
		    x + width > toIntersect.x &&
			y < toIntersect.y + toIntersect.height && 
			y + height > toIntersect.y);
}

bool Rectangle::contains(float x, float y) const
{
    return (x >= this->x && y >= this->y &&
            x <= this->x + width && y <= this->y + height);       
}

bool Rectangle::contains(const glm::vec2& point) const
{
    return contains(point.x, point.y);
}

bool Rectangle::contains(const Rectangle& rect) const
{
	return (rect.x >= x && rect.y >= y && 
			rect.x+rect.width <= x+width &&
			rect.y+rect.height <= y+height);
}

std::string Rectangle::toString()
{
    std::ostringstream oss;
    oss << "(x=" << x << ", y=" << y << ", w=" << width << ", h=" << height << ")";
    return oss.str();
}

bool Rectangle::operator==(const Rectangle &other) const
{
	return (other.x == this->x && other.y == this->y && 
		    other.width == this->width && other.height == this->height);
}

bool Rectangle::operator!=(const Rectangle &other) const
{
    return !(*this == other);
}