#ifndef Rag_Rectangle_h
#define Rag_Rectangle_h

#include <string>
#include <glm/glm.hpp>

namespace rag
{
	/// Represents a Rectangle.
    class Rectangle
    {
    
    public:
    
		/// Constructor with coordinates and size.
        Rectangle(float x = 0, float y = 0, float width = 0, float height = 0);
        
		/// Returns the union of the current instance with another Rectangle.
        Rectangle rectUnion(const Rectangle& toUnion) const;

		/// Returns the intersection of the current instance with another Rectangle.
        Rectangle intersection(const Rectangle& toIntersect) const;

		/// Returns true if the instance intersects with the other Rectangle.
		bool intersects(const Rectangle& toIntersect) const;
        
		/// Checks if a point lies inside the Rectangle.
        bool contains(float x, float y) const;

		/// Checks if a point lies inside the Rectangle.
		bool contains(const glm::vec2& point) const;

		/// Checks if a Rectangle lies inside the Rectangle.
		bool contains(const Rectangle& rect) const;

		bool operator==(const Rectangle &other) const;
        bool operator!=(const Rectangle &other) const;
        
		/// String representation of the Rectangle.
        std::string toString();

		float x, y, width, height;
        
    };
}

#endif
