#ifndef _RenderTarget_H_
#define _RenderTarget_H_

#include "Material.h"

namespace rag
{
	/// Color representation using 4 bytes.
	struct Color4B
	{
		unsigned char r;
		unsigned char g;
		unsigned char b;
		unsigned char a;
	};

	/// Vertex representation.
	struct Vertex
	{
		glm::vec2 position;
		glm::vec2 texCoords;
		Color4B color;
	};

	/// Render primitives.
	enum PrimitiveType
	{
		Points,
		Lines,
		LineLoop,
		LineStrip,
		Triangles,
		TriangleStrip,
		TriangleFan
	};

	typedef std::vector<Vertex> VertexArray;

/// Object where DisplayObject instances with render capability are suposed to render.
/** A RenderTarget is passed trough the Display List and provides functionality to
	enqueue render commands. RenderTarget will group commands that use similar Material
	in order to reduce the number of draw calls.
*/
class RenderTarget
{

public:

	/// Create a RenderTarget object.
	RenderTarget();

	/// Default destructor.
	virtual ~RenderTarget() {};

	/// Enqueues draw command.
	void draw(const VertexArray& vertexArray, const Material& material);
	
	/// Renders enqueued render commands.
	void flush();

private:

	Material currentMaterial;
	VertexArray batch;

};

} // rag

#endif
