#ifndef Rag_TTFFont_h
#define Rag_TTFFont_h

#include "ITextFont.h"
#include <rag/Platform.h>

#define STBTT_malloc(x,u)  malloc(x)
#define STBTT_free(x,u)    free(x)

#include "external/stb_truetype.h"
#include <string>
#include <map>
#include <glm/glm.hpp>

namespace rag
{
	/// Implementation of ITextFint based on TrueType or OpenType Fonts.
    class TTFFont: public ITextFont
    {
    public:
        
		/// Constructs a TTFFont using a path and a text size.
        TTFFont(const std::string& path, float pixelHeight = 24);
        virtual ~TTFFont();

		static void		addFontAlias(const std::string& alias, const std::string& fontPath);	///< Allows to use a different name (or an 'alias') to refer to a font.

		virtual int     getWidth(const std::string& text);								///< Returns the width of a text
		virtual void    print(const std::string& text, const glm::mat4& matrix);		///< Renders text. Asumes ortho projection 1:1 screen pixel.

		virtual void    setLetterSpacing(float value);									///< Sets the extra space between characters.
		virtual void    reloadTexture();												///< On context loss, reload textures. \TODO ITextFont should inherit Resource. Reload should be part of resource.
        
    private:
        
        void getBakedQuad(int char_index, float *xpos, float *ypos, stbtt_aligned_quad *q);
        
        /// To bake a new character that wasn't baked before.
        int bakeFontBitmap(int first_char, bool uploadTexture = true);

        void allocTexture();
        
        void uploadTexture();
        
        stbtt_fontinfo info;
        std::map<int, stbtt_bakedchar> chardata;
        float pixelHeight, scale;
        int tex_w, tex_h;
        GLuint textureName;
        unsigned char* buffer;
        int ascent, descent, lineGap;
        unsigned char* bmp;
        
        /// Cursors pointing to next character to bake.
        int x, y, bottom_y; // \todo refactor x and y names
        float ipw, iph;
        
        float letterSpacing;

		static std::map<std::string, std::string> fontsAlias;
        
    };
}

#endif
