#include "TextEdit.h"

using namespace rag;

#ifdef _WIN32
#define IDC_MAIN_EDIT	101
WNDPROC prevWndProc;
LRESULT CALLBACK EditWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	if (msg == WM_KEYDOWN)
	{
		if (wParam == VK_RETURN) {
			TextEdit* textEdit = (TextEdit*)GetWindowLongPtr(hWnd, GWLP_USERDATA);
			textEdit->close();
			return 0;
		}
	}

	return CallWindowProc(prevWndProc, hWnd, msg, wParam, lParam);
}
#endif

TextEdit* TextEdit::instance = NULL;

TextEdit::TextEdit(rag::DisplayObject* parent, const std::string& text) :
textValue(text),
handle(0)
{
	if (instance)
		instance->close(true);

	instance = this;

	// use the bounds of the parent to create a text box adjusted
	rag::Rectangle rect = parent->getBounds();

#ifdef _WIN32
	// Create an edit box
	HWND hWnd = (HWND)winutils.getWindow();
	HWND hEdit = CreateWindowEx(WS_EX_CLIENTEDGE,
								"EDIT",
								"",
								WS_CHILD|WS_VISIBLE|
								0,
								int(rect.x),
								int(rect.y),
								int(rect.width),
								int(rect.height),
								hWnd,
								(HMENU)IDC_MAIN_EDIT,
								GetModuleHandle(NULL),
								NULL);

	prevWndProc = (WNDPROC)SetWindowLongPtr(hEdit, GWLP_WNDPROC, (LONG_PTR)EditWndProc);
	SetWindowLongPtr(hEdit, GWLP_USERDATA, (LONG_PTR)this);

	HGDIOBJ hfDefault=GetStockObject(DEFAULT_GUI_FONT);
	SendMessage(hEdit,
				WM_SETFONT,
				(WPARAM)hfDefault,
				MAKELPARAM(FALSE,0));

	SendMessage(hEdit,
				WM_SETTEXT,
				NULL,
				(LPARAM)text.c_str());

	handle = (void*)hEdit;
#endif
}

void TextEdit::close(bool cancel)
{
#ifdef _WIN32
	char buffer[256];
	SendMessage((HWND)handle,
				WM_GETTEXT,
				sizeof(buffer)/sizeof(buffer[0]),
				reinterpret_cast<LPARAM>(buffer));

	std::string newValue = buffer;
	if (textValue != newValue && !cancel) {
		textValue = newValue;

		events::Event e(events::TEXTEDIT_FINISHED);
		e.target = this;
		dispatchEvent(e);
	}

	DestroyWindow((HWND)handle);
	handle = NULL;
#endif

	parent->deleteChild(this);
	instance = NULL;
}
