#include "TextEdit.h"

#ifdef __APPLE__

@interface TextEditDelegate : NSObject<UITextViewDelegate>

@end

@implementation TextEditDelegate

- (BOOL)textView:(UITextView *)textView shouldChangeTextInRange:(NSRange)range replacementText:(NSString *)text
{
	if ([text isEqualToString:@"\n"])
	{
        if (rag::TextEdit::getInstance())
            rag::TextEdit::getInstance()->close();
        return NO;
    }
    
    return YES;
}

- (void) textViewDidBeginEditing:(UITextView *)textView
{
}

- (void)textViewDidEndEditing:(UITextView *)textView
{
    if (rag::TextEdit::getInstance())
        rag::TextEdit::getInstance()->close();
}

@end

static TextEditDelegate* textEditDelegate = nil;

rag::TextEdit* rag::TextEdit::instance = NULL;

rag::TextEdit* rag::TextEdit::getInstance()
{
	return instance;
}

rag::TextEdit::TextEdit(rag::DisplayObject* parent, const std::string& text) :
textValue(text),
handle(0)
{
	if (instance)
		instance->close(true);

	instance = this;

	// use the bounds of the parent to create a text box adjusted
	rag::Rectangle bounds = parent->getBounds();

	CGRect rect = CGRectMake(bounds.x, bounds.y, bounds.width, bounds.height);
	UITextView* textView = [[UITextView alloc] initWithFrame:rect];
    textView.returnKeyType = UIReturnKeyDone;
	textView.autocorrectionType = UITextAutocorrectionTypeNo;
	//textView.textAlignment = UITextAlignmentLeft;
	textView.backgroundColor = [UIColor clearColor];
	textView.autocapitalizationType = UITextAutocapitalizationTypeNone;
	textView.text = [NSString stringWithUTF8String:text.c_str()];
	textView.backgroundColor = [UIColor whiteColor];

    if (textEditDelegate == nil)
        textEditDelegate = [[TextEditDelegate alloc] init];
    
	textView.delegate = textEditDelegate;

	UIView* view = (UIView*)iosUtils::getView();
	[view addSubview:textView];
	[textView becomeFirstResponder];
    [textView release];

	handle = (void*)textView;
}

void rag::TextEdit::close(bool cancel)
{
    instance = NULL;
	UITextView* textView = (UITextView*)handle;

	std::string newValue = textView.text.UTF8String;
	if (textValue != newValue && !cancel) {
		textValue = newValue;

		events::Event e(events::TEXTEDIT_FINISHED);
		e.target = this;
		dispatchEvent(e);
	}

	[textView resignFirstResponder];
    [textView removeFromSuperview];

	UIView* view = (UIView*)iosUtils::getView();
	[view endEditing:YES];

	parent->deleteChild(this);
}

#endif
