#ifndef _TEXTFIELD_H_
#define _TEXTFIELD_H_

#include <rag/DisplayObject.h>
#include <rag/ITextFont.h>

namespace rag
{
	/// Shadow effect for TextField instances.
    class DropShadowFilter
    {
        
    public:
        
        DropShadowFilter(float angle, float distance, float strength, Color color);
        
        Color color;
        float x, y;
    };
    
    /// High level abstraction to render texts in display list.
	class TextField: public DisplayObject
	{

	public:

         /// Constructor with Bitmap fonts.
		TextField(const std::string& folderPath, const std::string& descriptorFileName);
        
        /// Constructor with truetype fonts.
        TextField(const std::string& path, float pixelHeight = 24, float letterSpacing = 0);
        
        virtual ~TextField();

		void addFilter(DropShadowFilter filter);
		std::vector<DropShadowFilter> getFilters();

		/// Returns the number of lines used with the current text.
		int  getLines();

		/// Returns the height of a line.
		int getLineHeight();

		/// Returns the current length of the text, for single line.
		int getTextWidth();
        
        enum HorzAlignment { Left = 0, Center, Right } horzAlign; ///< The horizontal alignment of the text block.
		enum VertAlignment { Top = 0, Middle, Bottom } vertAlign; ///< The vertical alignment of the text block.
        
		std::string text;	///< The text that should be rendered.
        bool multiline;		///< Is it intended to be drawn in a single or multi-line fashion. False by default.
        bool showCursor;	///< When true, a cursor is shown right after the last letter. Note that text width remains the same with or without cursor.
        bool autotrim;		///< True by default. Trims single line text when longer than reserved dimensions.        
        bool password;		///< Use the textfield to show a password. Wildcards would be printed instead of the actual text.
        
		/// Debug function to know how many textures are cached by texts.
        static void traceTextCache();

		/// When graphic context is missed (android) reloads fonts textures.
        static void reloadTextures();

	protected:

        void                        init();

		virtual void                render() override;
        
        void                        printText(const std::string& text, const glm::mat4 matrix);
        
        std::vector<std::string>    splitLines();

        ITextFont*                  font;
        
        int                         lineHeight;
        
        // For single-line
        int                         lineWidth;
        
        // For multiline
        std::vector<std::string>    lines;
        std::vector<int>            lineWidths;
        
        std::string                 lastText;
        
        std::vector<DropShadowFilter> filters;
        
        float                       letterSpacing;

	private:

		static std::map<std::string, std::map<int, ITextFont*> > cache;
	};
}

#endif