#include "Timer.h"
#include <rag/Platform.h>
#include <chrono>
#include <thread>

#if defined(__linux__)
#include <unistd.h>
#endif


namespace rag
{
    void sleep(int ms)
    {
        std::this_thread::sleep_for(std::chrono::milliseconds(ms));
    }
}



//
// http://www.gnu.org/savannah-checkouts/gnu/libc/manual/html_node/Elapsed-Time.html
//
/* Subtract the `struct timeval' values X and Y,
 storing the result in RESULT.
 Return 1 if the difference is negative, otherwise 0.  */
// struct timeval *result, *x, *y;
int timeval_subtract (timeval* result, timeval* x, timeval* y);
int timeval_subtract (timeval* result, timeval* x, timeval* y)
{
    /* Perform the carry for the later subtraction by updating y. */
    if (x->tv_usec < y->tv_usec) {
        int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
        y->tv_usec -= 1000000 * nsec;
        y->tv_sec += nsec;
    }
    if (x->tv_usec - y->tv_usec > 1000000) {
        int nsec = (x->tv_usec - y->tv_usec) / 1000000;
        y->tv_usec += 1000000 * nsec;
        y->tv_sec -= nsec;
    }
    
    /* Compute the time remaining to wait.
     tv_usec is certainly positive. */
    result->tv_sec = x->tv_sec - y->tv_sec;
    result->tv_usec = x->tv_usec - y->tv_usec;
    
    /* Return 1 if result is negative. */
    return x->tv_sec < y->tv_sec;
}



// Win32 implementation of gettimeofday function mostly stolen from:
// http://www.suacommunity.com/dictionary/gettimeofday-entry.php
#ifdef _WIN32

struct timezone { int  tz_minuteswest; int  tz_dsttime; };

#include <time.h>
#include <windows.h>
#include <iostream>

#if defined(_MSC_VER) || defined(_MSC_EXTENSIONS)
#define DELTA_EPOCH_IN_MICROSECS  11644473600000000Ui64
#else
#define DELTA_EPOCH_IN_MICROSECS  11644473600000000ULL
#endif


// Definition of a gettimeofday function
int gettimeofday(struct timeval *tv, struct timezone *tz);
int gettimeofday(struct timeval *tv, struct timezone *tz)
{
	// Define a structure to receive the current Windows filetime
	FILETIME ft;
    
	// Initialize the present time to 0 and the timezone to UTC
	unsigned __int64 tmpres = 0;
	static int tzflag = 0;
    
	if (NULL != tv)
	{
		GetSystemTimeAsFileTime(&ft);
        
		// The GetSystemTimeAsFileTime returns the number of 100 nanosecond
		// intervals since Jan 1, 1601 in a structure. Copy the high bits to
		// the 64 bit tmpres, shift it left by 32 then or in the low 32 bits.
		tmpres |= ft.dwHighDateTime;
		tmpres <<= 32;
		tmpres |= ft.dwLowDateTime;
        
		// Convert to microseconds by dividing by 10
		tmpres /= 10;
        
		// The Unix epoch starts on Jan 1 1970.  Need to subtract the difference
		// in seconds from Jan 1 1601.
		tmpres -= DELTA_EPOCH_IN_MICROSECS;
        
		// Finally change microseconds to seconds and place in the seconds value.
		// The modulus picks up the microseconds.
		tv->tv_sec = (long)(tmpres / 1000000UL);
		tv->tv_usec = (long)(tmpres % 1000000UL);
	}
    
	if (NULL != tz)
	{
		if (!tzflag)
		{
			_tzset();
			tzflag++;
		}
        
		// Adjust for the timezone west of Greenwich
		tz->tz_minuteswest = _timezone / 60;
		tz->tz_dsttime = _daylight;
	}
    
	return 0;
}
#endif // _WIN32



using namespace rag;
using namespace std;

float Timer::deltaTime = 0;
float Timer::totalTime = 0;
float Timer::timeFactor = 1.0f;

Timer::Timer():
time(0),
loop(false),
startTime(0),
isRunning(false)
{
}

bool Timer::finished()
{
    return getDelta() >= 1;
}

bool Timer::running()
{
    return isRunning;
}

void Timer::start(float time, bool loop)
{
	this->time = time;
	this->loop = loop;
	startTime = rag::Timer::totalTime;
	isRunning = true;
}

void Timer::reset()
{
	isRunning = false;
    time = 0;
}


float Timer::getDelta(Ease::EaseType easetype)
{
	if (!isRunning) return 0.0f; // not started

	float fcur = rag::Timer::totalTime - startTime;
	
	if (fcur < 0)
	{ // This is a hack for dwarfs simulation moving time backwards (and should be avoided for other versions)
		startTime = rag::Timer::totalTime;
	}

	if (loop)
	{
		int cycles = (int)(fcur / time);
		fcur -= time * cycles;
	}

	ease.setType(easetype);
	return ease.get(fcur, time);
}









////////////////////////////////////////////////////////////////////////
// Chrono
////////////////////////////////////////////////////////////////////////
Chrono::Chrono()
{
    gettimeofday(&startTime, NULL);
}

float Chrono::getElapsedTime()
{
    timeval current;
    timeval elapsed;
    gettimeofday(&current, NULL);
    timeval_subtract(&elapsed, &current, &startTime);
    return elapsed.tv_sec + (float)elapsed.tv_usec / 1000000;
}

void Chrono::reset()
{
    gettimeofday(&startTime, NULL);
}

