#ifndef _TIMER_H_
#define _TIMER_H_

#include "Ease.h"
#include <rag/Platform.h>

namespace rag
{
    /// Provides time-related functionality.
	class Timer
	{

	public:

		Timer();

		/// Starts the Timer with a fixed amount of time.
		void        start(float time, bool loop = false);

		/// Returns the time elapsed since the start interpolated between 0 and 1.
		/** The interpolated value can use any curve, for convenience exposed in this function. */
		float       getDelta(Ease::EaseType easetype = Ease::linear_01);

		/// True if the timer has been running for the time specified at the start or more. Only valid for non-loop operation mode.
		bool        finished();

		/// True if the timer is running.
        bool        running();

		/// Reset stops the timer and puts it in the same state it was before start running.
        void        reset();
        
        /// Stores the elapsed time from frame to frame (use at your convenience).
        static float deltaTime;

		/// Stores total time since app starts.
		static float totalTime;

		/// Factor shared by which all Timer instances.
		static float timeFactor;

	private:

		float		time;
		bool		loop;
		float		startTime;
		bool		isRunning;
		rag::Ease	ease;


	};
    
    extern void sleep(int ms);
    
    
	/// Helper class to count time elapsed from a moment in time.
    class Chrono
    {
    public:
        Chrono();

		/// Get elapsed time from chrono construction
        float		getElapsedTime();

		/// Reset time to 0.
		void		reset();

	private:
        timeval 	startTime;
    };
}



#endif
