#ifndef Rag_XFLParser_h
#define Rag_XFLParser_h

#include <rag/DisplayObject.h>
#include <rag/MovieClip.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

namespace rag
{

/// Parser of XFL documents generated by the 3rd party editor tool FlashCS.
/** An XFLParser instance allows to read Flash symbols and creates the required 
	DisplayObject instances to reproduce a particular symbol. */
class XFLParser
{

public:

	/// Creates the parser object.
	XFLParser();

	/// Load a Flash CS symbol.
	rag::DisplayObject* load(const std::string& rootFolder, const std::string& symbol);

private:

	void parseRoot(xmlNode * root);
	void parseLayers(xmlNode* layers);
	void parseFrames(xmlNode * frames);
	void parseElements(xmlNode * elements);
	void parseScript(xmlNode * script);
	void parseSymbol(xmlNode * node);
	void parseBitmap(xmlNode * bitmap);
	void parseText(xmlNode* node);

	void applyMatrix(rag::DisplayObject* displayObject, xmlNode* matrix);
	void applyColor(rag::DisplayObject* displayObject, xmlNode* node);

	int         getAsInt(xmlNode* node, const char* property, int defaultValue = 0);
	float       getAsFloat(xmlNode* node, const char* property, float defaultValue = .0f);
	std::string getAsString(xmlNode* node, const char* property, std::string defaultValue = "");

	std::vector<xmlNode*> instanceLayers;
	rag::DisplayObject* instance;
	std::string         rootFolder;
	std::string			fileName;

	std::vector<rag::MovieClip::Frame> frames;


	// Binarization
	void				appendToBinary(float f);
	void				appendToBinary(int i);
	void				appendToBinary(char c);
	void				appendToBinary(const char* str);

	char*				binaryBuffer;
	int					binaryBufferSize;

	struct DropShadowFilterData
	{
		float angle;
		float distance;
		float strength;
		Color color;
	};



};

} // rag

#endif
