//
//  FileUtil.mm
//  dwarfs
//
//  Quick and *very* dirty methods to interact at a minimal level with file system from iOS.
//
//

#include <stdio.h>

#include "win/system/FileUtil.h"

#include <UIKit/UIKit.h>
#include <UIKit/UITableView.h>
#include <UIKit/UITableViewCell.h>
#include "iosUtils.h"
#include "core/Screen.h"
#include "core/Platform.h"

namespace fs
{
	class path
	{
        
	public:
        
		path(const std::string& name = ""):
        name(name)
		{
            
		}
        
		path parent_path()
		{
			size_t found = name.find_last_of('/');
			return name.substr(0, found);
		}
        
		bool has_parent_path()
		{
			return name.find('/') != std::string::npos;
		}
        
		bool has_extension()
		{
			size_t dot = name.find_last_of('.');
            
			// No dot, no extension
			if (dot == std::string::npos)
				return false;
            
			// Dot but not slash, there's extension
			size_t slash = name.find_last_of('/');
			if (slash == std::string::npos)
				return true;
			
			// Otherwise extension exists only if dot is after slash.
			return dot > slash;
		}
        
		void replace_extension(const std::string extension)
		{
			size_t dot = name.find_last_of('.');
			if (dot != std::string::npos)
			{
				// Replace previous extension
				name = name.substr(0, dot) + extension;
			}
			else
			{
				// Add a extension
				name = name + extension;
			}
		}
        
		std::string string() const
		{
			return name;
		}
        
		const path operator/(const path &rhs) const
		{
			return name + "/" + rhs.string();
		}
        
	private:
        
		std::string name;
        
	};
}

std::string currentPath = "assets/definitions/camps";

std::vector<std::string> getFilesInFolder(std::string folder)
{
    std::vector<std::string> answer;
    
    answer.push_back("..");
    
    std::string iosPath = rag::iosUtils::getResourcePath(folder);
    
    NSArray *directory = [[NSFileManager defaultManager] contentsOfDirectoryAtPath:[NSString stringWithUTF8String:iosPath.c_str()] error:NULL];
    for (NSString *item in directory){
        answer.push_back([item UTF8String]);
    }
    
    answer.push_back("[close]");
    return answer;
}


bool isDirectory(std::string path)
{
    fs::path p(path.c_str());
    return !p.has_extension();
    
    std::string iosPath = rag::iosUtils::getResourcePath(path);
    BOOL isDir;
    NSString* fullPath = [NSString stringWithUTF8String:iosPath.c_str()];
    [[NSFileManager defaultManager] fileExistsAtPath:fullPath isDirectory:&isDir];
    
    return isDir;
}

@interface MyTableView : UITableView<UITableViewDataSource, UITableViewDelegate> {
}

@end

@implementation MyTableView

#pragma mark - View lifecycle

- (void)loadView
{
    self.dataSource = self;
    self.delegate = self;
}

- (void)viewDidUnload {
}

#pragma mark - Table view data source

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    
    if (cell == nil) {
        
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
        
    }
    
    // Configure the cell...
    /*
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectoryPath = [paths objectAtIndex:0];
    NSFileManager *fm = [NSFileManager defaultManager];
    NSArray* dirContents = [fm contentsOfDirectoryAtPath:documentsDirectoryPath error:nil];
    cell.textLabel.text = [dirContents objectAtIndex:indexPath.row];
    */
    /*
    NSString *bundleRoot = [[NSBundle mainBundle] bundlePath];
    NSArray *directory = [[NSFileManager defaultManager] directoryContentsAtPath: bundleRoot];
    BOOL isDirectory;
    for (NSString *item in directory){
        BOOL fileExistsAtPath = [[NSFileManager defaultManager] fileExistsAtPath:rag::iosUtils::getResourcePath(<#const std::string &path#>) isDirectory:&isDirectory];
        if (fileExistsAtPath) {
            if (isDirectory)
            {
                //It's a Directory.
                cell.textLabel.text = item;
            }
        }
        if ([[item pathExtension] isEqualToString:@"png"]) {
            //This is Image File with .png Extension
        }
    }*/
    std::string file = getFilesInFolder(currentPath)[indexPath.row];
    if (isDirectory(file))
        file.append("/");
    
    
    cell.textLabel.text = [NSString stringWithUTF8String: file.c_str()];
    
    return cell;
    
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    /*
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectoryPath = [paths objectAtIndex:0];
    NSFileManager *fm = [NSFileManager defaultManager];
    NSArray* dirContents = [fm contentsOfDirectoryAtPath:documentsDirectoryPath error:nil];
     */
    
    return getFilesInFolder(currentPath).size();
    
    //return [dirContents count];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    NSLog(@"%@", indexPath);
    
    UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    
    std::string file =[cell.textLabel.text UTF8String];
    
    if (file == "[close]/")
    {
        [self removeFromSuperview];
    }
    else if (file == "..")
    {
        fs::path p(currentPath.c_str());
        if (p.has_parent_path())
        {
            // Move to parent dir
            currentPath = currentPath.substr(0, currentPath.size()-1);
            p = currentPath;
            currentPath = p.parent_path().string();
            [self removeFromSuperview];
            FileUtil::getInstance().startOpenFileDialog();
        }
    }
    // files in "islandXX" directories are files not folders.
    else if (isDirectory(file) && currentPath.find("island") == std::string::npos)
    {
        // Move to another directory
        currentPath += ("/" + file);
        [self removeFromSuperview];
        FileUtil::getInstance().startOpenFileDialog();
    }
    else
    {
        events::FileUtilEvent event(events::FILE_SELECTED);
        event.file = currentPath + file;
        event.fromBundle = true; // TODO not always!
        FileUtil::getInstance().dispatchEvent(event);
        
        [self removeFromSuperview];
    }
    
}



@end



void FileUtil::startOpenFileDialog()
{
    UIView *view = (UIView*)rag::iosUtils::getView();
    
    MyTableView* table = [[MyTableView alloc] initWithFrame:CGRectMake(0, 0, screen::width, screen::height) style:UITableViewStylePlain];
    [table loadView];
    table.dataSource = table;
    [view addSubview:table];
}

void FileUtil::startSaveFileDialog()
{
    events::FileUtilEvent event(events::FILE_SELECTED);
    event.file = "saveFile";
    FileUtil::getInstance().dispatchEvent(event);
}

