#import "ViewController.h"
#import <OpenGLES/EAGL.h>
#import <OpenGLES/EAGLDrawable.h>
#import <QuartzCore/QuartzCore.h>

#include "NativeApp.h"
#include "core/Screen.h"

#include "as3/events/TouchEvent.h"
#include "input/KeyboardManager.h"
#include "Keyboard.h"
#include "input/InputManager.h"

#include "iosUtils.h"
#import <Social/Social.h>
#import <sys/utsname.h>

#include "Alert.h"

#include "core/ResourceMgr.h"
#include "utils/ContentFile.h"

@interface ViewController () {
    
}

@property (strong, nonatomic) EAGLContext *context;

//- (void)setupScreen;
//- (void)setMaxScaleFactor: (float)maxScaleFactor;

@end

@implementation ViewController
@synthesize context = _context;
@synthesize keyboardType = UIKeyboardTypeDefault;

float scale = 1;


- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
// iOS 7.1    self.context.multiThreaded = YES;
    
    if (!self.context) {
        NSLog(@"Failed to create context");
    }
    
    GLKView *view = (GLKView*)self.view;
    view.context = self.context;
    view.drawableDepthFormat = GLKViewDrawableDepthFormat16;
    view.drawableColorFormat = GLKViewDrawableColorFormatRGB565;
    
    rag::iosUtils::setView(self.view);
    rag::iosUtils::setViewController(self);
    
    [self setupScreen];
    
    UIPinchGestureRecognizer* pinchGesture = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:@selector(handlePitchGesture:)];
    [self.view addGestureRecognizer:pinchGesture];
    [pinchGesture release];
    [self.view setMultipleTouchEnabled:YES];
    
    // register for keyboard notifications
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillShow:)
                                                 name:UIKeyboardWillShowNotification object:self.view.window];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillHide:)
                                                 name:UIKeyboardWillHideNotification object:self.view.window];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    
    [EAGLContext setCurrentContext:self.context];
    
    if ([EAGLContext currentContext] == self.context) {
        [EAGLContext setCurrentContext:nil];
    }
	self.context = nil;
}

- (BOOL)shouldAutorotate {
    
    return YES;
    
}

// For old iOS
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation {
    return  toInterfaceOrientation == UIInterfaceOrientationLandscapeLeft ||
            toInterfaceOrientation == UIInterfaceOrientationLandscapeRight;
}

- (NSUInteger)supportedInterfaceOrientations {
    
    return UIInterfaceOrientationMaskLandscape;
    
}

#pragma mark - GLKView and GLKViewController delegate methods

- (void)update
{
    GetNativeApp()->onLogicUpdate();
}

- (void)glkView:(GLKView *)view drawInRect:(CGRect)rect
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // move this to render (check cross-platform)
    GetNativeApp()->onRender();
    // usleep(100);
}

- (void)setupScreen
{
    // Read device info
    struct utsname u;
    uname(&u);
    
    std::string machine = u.machine;
    
    LOGI("machine: %s", machine.c_str());
    
    // Default scren values
    self.preferredFramesPerSecond = 60;
    
    // Customizable values for your particular game
    bool deviceFound = false;

    ContentFile contentFile;
    if (contentFile.load("devices.xml")) // Find in dynamic content
    {
        ContentFile::Row row;
        if (contentFile.findRowByValue("name", machine, row))
        {
            screen::isLowPerformer = contentFile.getRowValueAsInt(row, "low_performer");
            screen::isTablet = contentFile.getRowValueAsInt(row, "tablet");
            [self setMaxScaleFactor: contentFile.getRowValueAsFloat(row, "max_scale_factor")];
            deviceFound = true;
        }
    }
    else if (contentFile.load("assets/xml/devices.xml", true)) // Use default xml.
    {
        ContentFile::Row row;
        if (contentFile.findRowByValue("name", machine, row))
        {
            screen::isLowPerformer = contentFile.getRowValueAsInt(row, "low_performer");
            screen::isTablet = contentFile.getRowValueAsInt(row, "tablet");
            [self setMaxScaleFactor: contentFile.getRowValueAsFloat(row, "max_scale_factor")];
            deviceFound = true;
        }
    }
    if (!deviceFound)
    {
        // default values, and we can asume this is a newer device.
        [self setMaxScaleFactor:[[UIScreen mainScreen] scale]];
        screen::isLowPerformer = false;
        screen::isTablet = true;
    }
}

- (void) setMaxScaleFactor:(float)maxScaleFactor
{
    CGRect  rect = [[UIScreen mainScreen] bounds];
    scale = [[UIScreen mainScreen] scale];
    
    if (scale > maxScaleFactor && maxScaleFactor > 1.0f)
    {
        scale = maxScaleFactor;
    }
    
    screen::width = 1024;
    screen::height = 768;
    screen::realWidth = rect.size.width * scale;
    screen::realHeight = rect.size.height * scale;
    
    // if landscape
    if (screen::realWidth < screen::realHeight)
    {
        std::swap(screen::realWidth, screen::realHeight);
    }
    
    screen::center.x = screen::width / 2;
    screen::center.y = screen::height / 2;
    screen::scale = scale;
    self.view.contentScaleFactor = scale;
    
    float scaleX = (float)screen::realWidth / screen::width;
    float scaleY = (float)screen::realHeight / screen::height;
    screen::scaleBest = std::min(scaleX, scaleY);
    
    screen::factorX = screen::realWidth / (screen::width * screen::scaleBest);
    screen::factorY = screen::realHeight / (screen::height * screen::scaleBest);
    
    [EAGLContext setCurrentContext:self.context];
}







////////////////////////////////////////////////////////////////////////
// Input
////////////////////////////////////////////////////////////////////////

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    UITouch *touch = [touches anyObject];
    // startTouchPosition is an instance variable
    CGPoint startTouchPosition = [touch locationInView:self.view];
    
    events::TouchEvent e(events::TOUCH_BEGIN);
    e.stageX = startTouchPosition.x;
    e.stageY = startTouchPosition.y;
    InputManager::getInstance().processInputEvent(e);
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    UITouch *touch = [touches anyObject];
    CGPoint startTouchPosition = [touch locationInView:self.view];
    
    events::TouchEvent e(events::TOUCH_MOVE);
    e.stageX = startTouchPosition.x;
    e.stageY = startTouchPosition.y;
    
    InputManager::getInstance().processInputEvent(e);
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    UITouch *touch = [touches anyObject];
    
    if ([touch phase] != UITouchPhaseEnded && [touch phase] != UITouchPhaseCancelled) // This prevents cancel phase events
        return;
    
    CGPoint startTouchPosition = [touch locationInView:self.view];
        
    events::TouchEvent e(events::TOUCH_END);
    e.stageX = startTouchPosition.x;
    e.stageY = startTouchPosition.y;
    InputManager::getInstance().processInputEvent(e);
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
    [self touchesEnded:touches withEvent:event];
}

- (IBAction)handlePitchGesture:(UIPinchGestureRecognizer*)sender
{
    CGFloat factor = [sender scale];
    CGFloat velocity = [sender velocity];
    
    if (velocity != velocity)
        return; // Check for NaN
    
    if ([sender numberOfTouches] < 2)
        return;
    
    // Pinch center
    CGPoint p1 = [sender locationOfTouch:0 inView:self.view];
    CGPoint p2 = [sender locationOfTouch:1 inView:self.view];
    
    float cx = (p2.x - p1.x)/2.0 + p1.x;
    float cy = (p2.y - p1.y)/2.0 + p1.y;
    
    events::TouchEvent e(events::TOUCH_PINCH);
    e.pinch = velocity;
    e.stageX = cx;
    e.stageY = cy;
    InputManager::getInstance().processInputEvent(e);
    
    ((UIPinchGestureRecognizer*)sender).scale = 1;
}






////////////////////////////////////////////////////////////////////////
// Soft Keyboard
////////////////////////////////////////////////////////////////////////
- (BOOL)canBecomeFirstResponder {
    return rag::iosUtils::requestingKeyboard();
}

- (void)insertText:(NSString *)text {
    // Do something with the typed character
    rag::KeyboardManager::getInstance().nativeInsertText([text UTF8String]);
}
- (void)deleteBackward {
    // Handle the delete key
    rag::KeyboardManager::getInstance().nativeDeleteBackward();
}
- (BOOL)hasText {
    // Return whether there's any text present
    return YES;
}
- (void)keyboardWillShow:(NSNotification*)aNotification
{    
    NSDictionary* info = [aNotification userInfo];
    CGSize kbSize = [[info objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue].size;
    Keyboard::size.width = kbSize.width;
    Keyboard::size.height = kbSize.height;
}

- (void)keyboardWillHide:(NSNotification*)aNotification
{
    events::Event e(events::KEYBOARD_WILL_HIDE);
    rag::KeyboardManager::getInstance().dispatchEvent(e);
}

- (UIKeyboardType) keyboardType
{
    return keyboardType;
}

- (void) setKeyboardType:(UIKeyboardType)_keyboardType
{
    keyboardType = _keyboardType;
}

-(void) didReceiveMemoryWarning
{
    rag::resourcemgr.memoryWarnings++;
}

/*
- (void)postTwit:(NSString *)twit
{
    if ([SLComposeViewController isAvailableForServiceType:SLServiceTypeTwitter])
    {
        SLComposeViewController *tweetSheet = [SLComposeViewController composeViewControllerForServiceType:SLServiceTypeTwitter];
        [tweetSheet setInitialText:twit];
        [self presentViewController:tweetSheet animated:YES completion:nil];
    }
    else
    {
        new rmm::Alert("Sorry", "You can't send a tweet right now, make sure your device has an internet connection and you have at least one Twitter account setup", "OK");
    }
}*/

@end
