#include "iosUtils.h"
#import "ViewController.h"
#include <map>
#import "UICKeyChainStore.h"
#import <sys/utsname.h>
//	#include "ui/Alert.h"

using namespace rag;
using namespace std;

static UIView* view = NULL;
static ViewController* viewController = NULL;
bool   wantsKeyboard = false;
bool iosUtils::logEnabled = true;
std::string iosUtils::deviceModel;

string iosUtils::getResourcePath(const string& path)
{
    NSString *nsPath = [NSString stringWithUTF8String:path.c_str()];
    NSString *resourcepath = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:nsPath];
    
    return [resourcepath cStringUsingEncoding:NSUTF8StringEncoding];
}

string iosUtils::getSaveFolder(const string& path)
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectoryPath = [paths objectAtIndex:0];
    NSString* nsPath = [NSString stringWithUTF8String:path.c_str()];
    NSString *url = [documentsDirectoryPath stringByAppendingPathComponent:nsPath];
    
    return [url cStringUsingEncoding:NSUTF8StringEncoding];
}

void iosUtils::showKeyboard()
{
    wantsKeyboard = true;
    [viewController becomeFirstResponder];
}

void iosUtils::hideKeyboard()
{
    wantsKeyboard = false;
    [viewController resignFirstResponder];
}

void iosUtils::setKeyboardType(KeyboardType type)
{
    map<KeyboardType, UIKeyboardType> m;
    m[KeyboardTypeDefault] = UIKeyboardTypeDefault;
    m[KeyboardTypeEmail] = UIKeyboardTypeEmailAddress;
    
    [viewController setKeyboardType:m[type]];
}

void iosUtils::setView(void* _view)
{
    view = (UIView*)_view;
}

void* iosUtils::getView()
{
    return view;
}

void iosUtils::setViewController(void* _view)
{
    viewController = (ViewController*)_view;
}

bool iosUtils::requestingKeyboard()
{
    return wantsKeyboard;
}

string iosUtils::getAppVersion()
{
    return [[[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"] UTF8String];
}

string iosUtils::getLanguage()
{
    NSString * language = [[NSLocale preferredLanguages] objectAtIndex:0];
    return [language UTF8String];
}

string iosUtils::toUpperUTF8(std::string s)
{
    NSString* ns = [NSString stringWithUTF8String:s.c_str()];
    ns = [ns uppercaseString];
    // NSLocale *locale = [[[NSLocale alloc] initWithLocaleIdentifier:@"en-US"] autorelease];
    // ns = [ns uppercaseStringWithLocale:locale];
    // ns = [ns stringByFoldingWithOptions:NSDiacriticInsensitiveSearch locale:locale];
    
    
    return [ns UTF8String];
}

string iosUtils::toLowerUTF8(std::string s)
{
    NSString* ns = [NSString stringWithUTF8String:s.c_str()];
    ns = [ns lowercaseString];
    return [ns UTF8String];
}

void iosUtils::postTweet(const std::string& defaultText)
{
    NSString* nsTwit = [[NSString alloc] initWithCString:defaultText.c_str() encoding:NSUTF8StringEncoding];
    [viewController postTwit:nsTwit];
}

std::string iosUtils::getNewUID()
{
    //first check if data is already saved on the keychan
    NSString* storedId = [UICKeyChainStore stringForKey:@"userDeviceId" service:@"com.dwarfsAndDragons"];
    if (storedId.length > 2) {
        return [storedId UTF8String];
    }
    
    //if not, get it and save it on keychan
    //ios 6
    if ([[UIDevice currentDevice] respondsToSelector:@selector(identifierForVendor)])
    {
        NSString* ns = [[[UIDevice currentDevice] identifierForVendor] UUIDString];
        [UICKeyChainStore setString:ns forKey:@"userDeviceId" service:@"com.dwarfsAndDragons"];
        return [ns UTF8String];
    }
    
    //pre ios 6
    NSString* uniqueIdentifier = nil;
    uniqueIdentifier = [[NSUserDefaults standardUserDefaults] objectForKey:@"identiferForVendor"];
    if( !uniqueIdentifier ) {
        CFUUIDRef uuid = CFUUIDCreate(NULL);
        uniqueIdentifier = (__bridge_transfer NSString*)CFUUIDCreateString(NULL, uuid);
        CFRelease(uuid);
        [[NSUserDefaults standardUserDefaults] setObject:uniqueIdentifier forKey:@"identifierForVendor"];
        [UICKeyChainStore setString:uniqueIdentifier forKey:@"userDeviceId" service:@"com.dwarfsAndDragons"];
    }
    
    return [uniqueIdentifier UTF8String];
}

std::string iosUtils::getIslandID()
{
#ifndef FINAL_RELEASE
    NSString* ns = [[NSUserDefaults standardUserDefaults] stringForKey:@"userid_island"];
    if(ns == nil)
        ns = @"0";
#else
    NSString* ns = @"0";
#endif
    return [ns UTF8String];
}

std::string iosUtils::getUserID()
{
    NSString* ns = [[NSUserDefaults standardUserDefaults] stringForKey:@"userid_preference"];
    if(ns == nil)
        ns = @"0";
    return [ns UTF8String];
}

std::string iosUtils::getUserName()
{
    NSString* ns = [[NSUserDefaults standardUserDefaults] stringForKey:@"userid_name"];
    if(ns == nil)
        ns = @"No user";
    return [ns UTF8String];
}

void iosUtils::scheduleNotification(std::string _sku, std::string _body, int timeLeft, bool forceDayHours)
{
    NSString* sku = [NSString stringWithUTF8String:_sku.c_str()];
    NSString* body = [NSString stringWithUTF8String:_body.c_str()];
    
    NSDate* fireDate = [NSDate dateWithTimeIntervalSinceNow:timeLeft];
    //NSDate* fireDateNow = [NSDate dateWithTimeIntervalSinceNow:0];
    
    if (forceDayHours) {
        NSCalendar *gregorian14H = [[NSCalendar alloc] initWithCalendarIdentifier: NSGregorianCalendar];
        NSDateComponents *components14H = [gregorian14H components: NSUIntegerMax fromDate: fireDate]; //NSUIntegerMax doesn't work if we set year
        [components14H setHour: 14];
        [components14H setMinute:0];
        [components14H setSecond:0];
        NSDate *newDate14H = [gregorian14H dateFromComponents: components14H];
        
        NSCalendar *gregorian20H = [[NSCalendar alloc] initWithCalendarIdentifier: NSGregorianCalendar];
        NSDateComponents *components20H = [gregorian20H components: NSUIntegerMax fromDate: fireDate]; //NSUIntegerMax doesn't work if we set year
        [components20H setHour: 20];
        [components20H setMinute:0];
        [components20H setSecond:0];
        NSDate *newDate20H = [gregorian20H dateFromComponents: components20H];
        
        [gregorian20H release];
        [gregorian14H release];
        
        if ([fireDate compare:newDate14H] == NSOrderedAscending) { //fire date is before 14h, so we set it at 14
            fireDate = newDate14H;
        }
        else if ([fireDate compare:newDate20H] == NSOrderedDescending) { //fire date is after 20h, so we set it at 14
            fireDate = newDate14H;
        }
    }
    
    // Cancel any previouly scheduled notification with the same key
    cancelNotification(_sku);
    
    UILocalNotification* localNotif = [[UILocalNotification alloc] init];
    if (localNotif == nil) return;
    
    localNotif.fireDate = fireDate;
    localNotif.alertBody = body;
    localNotif.alertAction = @"Resume game";
    
    localNotif.soundName = UILocalNotificationDefaultSoundName;
    localNotif.applicationIconBadgeNumber = 1;
    
    localNotif.userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
                           sku, @"sku",
                           nil];
    
    [[UIApplication sharedApplication] scheduleLocalNotification:localNotif];
    
    [localNotif release];
}

void iosUtils::cancelNotification(std::string skustr)
{
    //if (isLocalNotificationsEnabled)
    {
        NSString* sku = [NSString stringWithUTF8String:skustr.c_str()];
        UIApplication* app = [UIApplication sharedApplication];
        for (UILocalNotification* localNotif in app.scheduledLocalNotifications)
        {
            NSString* skuStr = [localNotif.userInfo valueForKey:@"sku"];
            if (skuStr && [skuStr isEqualToString:sku])
            {
                [app cancelLocalNotification:localNotif];
                return;
            }
        }
    }
}

void iosUtils::openExternalURL(std::string urlString)
{
    NSString* urlStringNS = [NSString stringWithUTF8String:urlString.c_str()];
    [[UIApplication sharedApplication] openURL:[NSURL URLWithString:urlStringNS]];
}

void iosUtils::activatePushNotifications()
{
   
    UIApplication *application = [UIApplication sharedApplication];
    if ([application respondsToSelector:@selector(registerUserNotificationSettings:)]) { //ios >= 8
        UIUserNotificationSettings *settings = [UIUserNotificationSettings settingsForTypes:(UIUserNotificationTypeBadge |
                                                                                             UIUserNotificationTypeSound |
                                                                                             UIUserNotificationTypeAlert) categories:nil];
        [application registerUserNotificationSettings:settings];
    } else { //ios < 8
        UIRemoteNotificationType myTypes = UIRemoteNotificationTypeBadge | UIRemoteNotificationTypeAlert | UIRemoteNotificationTypeSound;
        [application registerForRemoteNotificationTypes:myTypes];
    }
    
}

//#define LOG_TO_FILE
#ifdef LOG_TO_FILE
    #include "core/File.h"
#endif
void iosUtils::iosLog(const char* format, ...)
{
#if defined(FINAL_RELEASE) && !defined(IS_STAGE)
    return;
#endif
    
#ifdef LOG_TO_FILE
    
    char buffer[2048];
    
    va_list argptr;
    va_start(argptr, format);
    vsprintf (buffer, format, argptr);
    va_end(argptr);
    
    rag::File file("log.txt", false, false);
    file.open("a");
    std::string strbuf = std::string(buffer) + "\n";
    file.write(strbuf.data(), 1, strbuf.size());
    file.close();
    
#else
    
    if (!logEnabled)
        return;
        
    va_list argptr;
    va_start(argptr, format);
    vfprintf(stderr, format, argptr);
    va_end(argptr);
    fprintf(stderr, "%s", "\n");

#endif
}



void iosUtils::iosLogError(const char* format, ...)
{
#if defined(FINAL_RELEASE) && !defined(IS_STAGE)
    return;
#endif
    
#ifdef LOG_TO_FILE
    
    char buffer[2048];
    
    va_list argptr;
    va_start(argptr, format);
    vsprintf (buffer, format, argptr);
    va_end(argptr);
    
    rag::File file("log.txt", false, false);
    file.open("a");
    std::string strbuf = std::string(buffer) + "\n";
    file.write(strbuf.data(), 1, strbuf.size());
    file.close();
    
#else
    
    if (!logEnabled)
        return;
    
    char buffer[2048];
    
    va_list argptr;
    va_start(argptr, format);
    vsprintf (buffer, format, argptr);
    va_end(argptr);
    fprintf(stderr, "%s", "\n");
    
    // new Alert("ERROR", std::string(buffer), "ok");
    
#endif
}

void iosUtils::iosLogEnable(bool value)
{
    logEnabled = value;
}

void iosUtils::sendErrorMail(std::string buildVersionS, std::string playerIDS)
{
    NSString* mailSubject = @"Dwarfs and Dragons feedback";
    
    NSString* gameName = @"Dwarfs and Dragons";
    NSString* buildVersion = [NSString stringWithUTF8String:buildVersionS.c_str()];
    NSString* playerID = [NSString stringWithUTF8String:playerIDS.c_str()];
    struct utsname u;
    uname(&u);
    std::string machine = u.machine;
    NSString* device = [NSString stringWithUTF8String:machine.c_str()];
    NSString* iOSVersion = [[UIDevice currentDevice] systemVersion];
    NSString *dateString = [NSDateFormatter localizedStringFromDate:[NSDate date]
                                                          dateStyle:NSDateFormatterShortStyle
                                                          timeStyle:NSDateFormatterFullStyle];
    
    NSMutableString* footer = [NSMutableString string];
    
    [footer appendString:@"---------- DO NOT DELETE THIS ----------\n"];
    [footer appendString:[NSString stringWithFormat:@"%@\n", dateString]];
    [footer appendString:[NSString stringWithFormat:@"%@\n", gameName]];
    [footer appendString:[NSString stringWithFormat:@"Version: %@\n", buildVersion]];
    [footer appendString:[NSString stringWithFormat:@"Player ID: %@\n", playerID]];
    [footer appendString:[NSString stringWithFormat:@"Device: %@\n", device]];
    [footer appendString:[NSString stringWithFormat:@"iOS: %@\n", iOSVersion]];
    [footer appendString:@"-----------------------------------------------------\n"];
    
    NSString* mailBody = @"";
    mailBody = [mailBody stringByAppendingFormat:@"\n\n\n%@", footer];
    
    NSString* mailTo = @"dwarfs.feedback@omnidrone.net";
    
    NSString *mailToFormatted = [NSString stringWithFormat:@"mailto:%@?subject=%@&body=%@",
                                 [mailTo stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding],
                                 [mailSubject stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding],
                                 [mailBody  stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]];
    
    [[UIApplication sharedApplication] openURL:[NSURL URLWithString:mailToFormatted]];
    
}

#import <AdSupport/ASIdentifierManager.h>

std::string iosUtils::getIDFA()
{
    NSString *nssIdfa = nil;
    if([[ASIdentifierManager sharedManager] isAdvertisingTrackingEnabled])
    {
        NSUUID *IDFA = [[ASIdentifierManager sharedManager] advertisingIdentifier];
        
        nssIdfa = [IDFA UUIDString];
        return [nssIdfa UTF8String];
    }
    return "";
}

