#ifndef ragnar_Descriptor_h
#define ragnar_Descriptor_h

#include <rag/Color.h>
#include <rag/Rectangle.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

namespace particles
{
    class Descriptor
    {
        
    public:
        
        Descriptor();
		Descriptor(const std::string& file);
		void save(const std::string file);
		void scale(float scale);
        
        // System parameters                                        
        // ------------------------------------------------------------------------
		enum SystemType
		{
			Continuous = 0,
			Timed,
			Explosive,
			LowPaced,

			NumSystemTypes
		} systemType;

        float systemLifeTime; // Only for timed systemType.

		int lowPaceEmissionMin; // Only for LowPaced  systemType
		int lowPaceEmissionMax; // Only for LowPaced  systemType

		int emission;
		bool additive;
        
        float particleLifeTimeMax;  
        float particleLifeTimeMin;  
        
		enum SpawnType
		{
		  STPoint = 0,
		  STOvalArea,
		  STRectangleArea
		};
		SpawnType		spawnType;
		float			spawnAreaWidth;
		float			spawnAreaHeight;

		bool animated;
		int  fps;
		bool firstFrameRandom;
        
        // Movement parameters
        // ------------------------------------------------------------------------
        float radialAccelerationMax;     
        float radialAccelerationMin;     
        
        glm::vec2 gravityNormDir;
        float gravityMax;                       
        float gravityMin;                       
        
        glm::vec2 velocityNormDir;
        float velocityMax;                      
        float velocityMin;                      
        
        float spread;                           
        
        
        // Particle appareance
        // ------------------------------------------------------------------------
        std::string file;
        float scaleStart;                      
        float scaleEnd;                        
        float scaleVar;                        
        
        rag::Color colorStart;                      
        rag::Color colorEnd;                        
        
		bool  orientated;
        float spinStart;                       
        float spinEnd;                         
        float spinVar;     

		static void clearCache();

	private:
		void parseElements(xmlNode* a_node);

    };
    
    
}

#endif
