#ifdef _WIN32

#include "Editor.h"
#include <AntTweakBar.h>
#include <rag/Screen.h>
#include "win/system/FileUtil.h"
#include "shared/utils/TextUtils.h"
#include <rag/Renderer.h>

static int backgroundColor = 0xFF000000;

using namespace particles;

EmitterEditor::FileOperation EmitterEditor::curretFileOperation = None;

EmitterEditor::EmitterEditor():
colorStart(0xFFFFFFFF),
colorEnd(0xFFFFFFFF),
direction(0.0f, 1.0f, 0.0f),
particleCount(0)
{
	clientData[CD_Close] = ClientData(this, this, "close");
	clientData[CD_ChooseImage] = ClientData(this, this, "choose_image");
	clientData[CD_Save] = ClientData(this, this, "save");
	clientData[CD_SaveAs] = ClientData(this, this, "saveas");
	clientData[CD_Load] = ClientData(this, this, "load");
	clientData[CD_MoveToTop] = ClientData(this, this, "move_to_top");

	static int emitterNumber = 0;
	std::ostringstream oss;
	oss << "Emitter_" << emitterNumber++;

	this->name = oss.str();

	bar = TwNewBar(oss.str().c_str());
	
	std::ostringstream conf;
	conf << oss.str() << " size='240 750'";
	TwDefine(conf.str().c_str());

	TwAddButton(bar, "close", Callback, &clientData[CD_Close], "");
	TwAddButton(bar, "move to top", Callback, &clientData[CD_MoveToTop], "");
	TwAddVarRO(bar, "particles", TW_TYPE_INT32, &particleCount, NULL);
	TwAddSeparator(bar, NULL, NULL);

	TwAddButton(bar, "choose_image", Callback, &clientData[CD_ChooseImage], "");	
	TwAddVarRW(bar, "emission", TW_TYPE_INT32, &descriptor.emission, "min=0 max=500");

	TwEnumVal systemTypeEV[] = { {particles::Descriptor::Continuous, "Continuous"}, {particles::Descriptor::Timed, "Timed"}, {particles::Descriptor::Explosive, "Explosive"}, {particles::Descriptor::LowPaced, "LowPaced"} };
	TwType systemType = TwDefineEnum("SystemType", systemTypeEV, particles::Descriptor::NumSystemTypes);
	TwAddVarRW(bar, "system type", systemType, &descriptor.systemType, NULL);

	TwAddVarRW(bar, "life_time", TW_TYPE_FLOAT, &descriptor.systemLifeTime, "min=0 max=5 precision=2 step=0.01");

	TwAddVarRW(bar, "time_emission_min", TW_TYPE_INT32, &descriptor.lowPaceEmissionMin, "min=1");
	TwAddVarRW(bar, "time_emission_max", TW_TYPE_INT32, &descriptor.lowPaceEmissionMax, "min=1");

	TwAddVarRW(bar, "additive", TW_TYPE_BOOLCPP, &descriptor.additive, "");
	TwAddVarRW(bar, "direction", TW_TYPE_DIR3F, &direction, "");

	TwAddSeparator(bar, NULL, NULL);

	TwAddVarRW(bar, "animated", TW_TYPE_BOOLCPP, &descriptor.animated, "");
	TwAddVarRW(bar, "frame 1 random", TW_TYPE_BOOLCPP, &descriptor.firstFrameRandom, "");	
	TwAddVarRW(bar, "fps", TW_TYPE_INT32, &descriptor.fps, "min=0 max=120");

	TwAddSeparator(bar, NULL, NULL);
	
	TwEnumVal spawnEV[] = { {particles::Descriptor::STPoint, "Point"}, {particles::Descriptor::STOvalArea, "Oval Area"}, {particles::Descriptor::STRectangleArea, "Rectangle Area"} };
	TwType spawnType = TwDefineEnum("SpawnType", spawnEV, 3);
	TwAddVarRW(bar, "spawn area", spawnType, &descriptor.spawnType, NULL);
	TwAddVarRW(bar, "width", TW_TYPE_FLOAT, &descriptor.spawnAreaWidth, "min=0");
	TwAddVarRW(bar, "height", TW_TYPE_FLOAT, &descriptor.spawnAreaHeight, "min=0");
	
	TwAddSeparator(bar, NULL, NULL);
	TwAddVarRW(bar, "max p life", TW_TYPE_FLOAT, &descriptor.particleLifeTimeMax, "min=0 max=20 precision=2 step=0.01");
	TwAddVarRW(bar, "min p life", TW_TYPE_FLOAT, &descriptor.particleLifeTimeMin, "min=0 max=20 precision=2 step=0.01");

	TwAddSeparator(bar, NULL, NULL);
	TwAddVarRW(bar, "max p radial acceleration", TW_TYPE_FLOAT, &descriptor.radialAccelerationMax, "min=-2.5 max=2.5 precision=4 step=0.0001");
	TwAddVarRW(bar, "min p radial acceleration", TW_TYPE_FLOAT, &descriptor.radialAccelerationMin, "min=-2.5 max=2.5 precision=4 step=0.0001");

	TwAddSeparator(bar, NULL, NULL);
	TwAddVarRW(bar, "max p gravity", TW_TYPE_FLOAT, &descriptor.gravityMax, "min=-5000 max=5000");
	TwAddVarRW(bar, "min p gravity", TW_TYPE_FLOAT, &descriptor.gravityMin, "min=-5000 max=5000");

	TwAddSeparator(bar, NULL, NULL);
	TwAddVarRW(bar, "max p velocity", TW_TYPE_FLOAT, &descriptor.velocityMax, "min=0 max=10000");
	TwAddVarRW(bar, "min p velocity", TW_TYPE_FLOAT, &descriptor.velocityMin, "min=0 max=10000");

	TwAddSeparator(bar, NULL, NULL);
	TwAddVarRW(bar, "spread", TW_TYPE_FLOAT, &descriptor.spread, "min=0 max=360");

	TwAddSeparator(bar, NULL, NULL);
	TwAddVarRW(bar, "scale start", TW_TYPE_FLOAT, &descriptor.scaleStart, "min=0 max=20 precision=2 step=0.01");
	TwAddVarRW(bar, "scale end", TW_TYPE_FLOAT, &descriptor.scaleEnd, "min=0 max=20 precision=2 step=0.01");
	TwAddVarRW(bar, "scale random", TW_TYPE_FLOAT, &descriptor.scaleVar, "min=0 max=20 precision=2 step=0.01");

	TwAddSeparator(bar, NULL, NULL);
	TwAddVarRW(bar, "orientated", TW_TYPE_BOOL32, &descriptor.orientated, "");
	TwAddVarRW(bar, "spin start", TW_TYPE_FLOAT, &descriptor.spinStart, "min=-20 max=20 precision=2 step=0.01");
	TwAddVarRW(bar, "spin end", TW_TYPE_FLOAT, &descriptor.spinEnd, "min=-20 max=20 precision=2 step=0.01");
	TwAddVarRW(bar, "spin random", TW_TYPE_FLOAT, &descriptor.spinVar, "min=-5 max=5 precision=2 step=0.01");

	TwAddSeparator(bar, NULL, NULL);
	TwAddVarRW(bar, "color start", TW_TYPE_COLOR32, &colorStart, "coloralpha=true");
	TwAddVarRW(bar, "color end", TW_TYPE_COLOR32, &colorEnd, "coloralpha=true");

	TwAddSeparator(bar, NULL, NULL);
	TwAddButton(bar, "save as", Callback, &clientData[CD_SaveAs], "");
	TwAddButton(bar, "save", Callback, &clientData[CD_Save], "");
	TwAddButton(bar, "load", Callback, &clientData[CD_Load], "");

	this->width = (float)rag::Screen::width;
	this->height = (float)rag::Screen::height;
	addEventListener(events::TOUCH_BEGIN, this);
	addEventListener(events::TOUCH_MOVE, this);

	loadEmitter("assets/particles/particle_editor.xml");

	path = rag::fs::path("");
	
	emitter = new particles::Emitter(descriptor);
	addChild(emitter);

	WinUtils::getInstance().addEventListener(events::WIN_EVENT, this);
}

EmitterEditor::~EmitterEditor()
{
	TwDeleteBar(bar);
	bar = NULL;
	WinUtils::getInstance().removeEventListener(events::WIN_EVENT, this);
}

void EmitterEditor::saveas()
{
	FileUtil::getInstance().addEventListener(events::FILE_SELECTED, this);
	curretFileOperation = SaveEmitter;
	FileUtil::getInstance().startSaveFileDialog();
}

void EmitterEditor::save()
{
	if (path.string().empty())
	{
		saveas();
	}
	else
	{
		descriptor.save(path.string());
	}
}

void TW_CALL EmitterEditor::Callback(void *clientData)
{ 
	if (!clientData)
		return;

	ClientData* data = (ClientData*)(clientData);
	
	if (data->operation == "saveas")
	{
		EmitterEditor* editor = static_cast<EmitterEditor*>(data->editor);
		editor->saveas();
	}
	else if (data->operation == "save")
	{
		EmitterEditor* editor = static_cast<EmitterEditor*>(data->editor);
		editor->save();
	}
	else if (data->operation == "load")
	{
		FileUtil::getInstance().addEventListener(events::FILE_SELECTED, data->listener);
		curretFileOperation = LoadEmitter;
		FileUtil::getInstance().startOpenFileDialog();
	}
	else if (data->operation == "choose_image")
	{
		FileUtil::getInstance().addEventListener(events::FILE_SELECTED, data->listener);
		curretFileOperation = ChooseImage;
		FileUtil::getInstance().startOpenFileDialog();
	}
	else if (data->operation == "close")
	{
		data->editor->parent->deleteChild(data->editor);
	}
	else if (data->operation == "move_to_top")
	{
		data->editor->parent->addChild(data->editor);
	}
}

void EmitterEditor::logicUpdate()
{
	{
		int a = (colorStart >> 24) & 0xFF;
		int b = (colorStart >> 16) & 0xFF;
		int g = (colorStart >>  8) & 0xFF;
		int r = (colorStart >>  0) & 0xFF;
		descriptor.colorStart = rag::Color(r/255.0,g/255.0,b/255.0,a/255.0);
	}
	{
		int a = (colorEnd >> 24) & 0xFF;
		int b = (colorEnd >> 16) & 0xFF;
		int g = (colorEnd >>  8) & 0xFF;
		int r = (colorEnd >>  0) & 0xFF;
		descriptor.colorEnd = rag::Color(r/255.0,g/255.0,b/255.0,a/255.0);
	}
	{
		int a = (backgroundColor >> 24) & 0xFF;
		int b = (backgroundColor >> 16) & 0xFF;
		int g = (backgroundColor >>  8) & 0xFF;
		int r = (backgroundColor >>  0) & 0xFF;
		rag::Color color(r/255.0,g/255.0,b/255.0,a/255.0);
		renderer.setClearColor(color);
	}

	if (glm::length(direction) == 0)
	{
		direction = glm::vec3(0.0f, 1.0f, 0.0f);
	}

	direction[2] = 0;
	direction = glm::normalize(direction);
	
	descriptor.velocityNormDir.x = direction.x;
	descriptor.velocityNormDir.y = -direction.y;
	
	emitter->setDescriptor(descriptor);
	if (descriptor.systemType != lastSystemType)
	{
		lastSystemType = descriptor.systemType;
		emitter->fire();
	}

	particleCount = emitter->getParticleCount();

	{
	std::ostringstream oss;
	oss << this->name << "/" << "life_time visible=" << (descriptor.systemType == Descriptor::Timed ? "true" : "false");
	TwDefine(oss.str().c_str());
	}
	{
	std::ostringstream oss;
	oss << this->name << "/" << "time_emission_min visible=" << (descriptor.systemType == Descriptor::LowPaced ? "true" : "false");
	TwDefine(oss.str().c_str());
	}
	{
	std::ostringstream oss;
	oss << this->name << "/" << "time_emission_max visible=" << (descriptor.systemType == Descriptor::LowPaced ? "true" : "false");
	TwDefine(oss.str().c_str());
	}
}

void EmitterEditor::onEvent(const std::string& type, events::Event& event)
{
	if (type == events::TOUCH_BEGIN)
	{
		LOGI("fire");
		emitter->fire();
		events::TouchEvent&e = static_cast<events::TouchEvent&>(event);
		emitter->setPosition(e.stageX, e.stageY);
	}
	if (type == events::TOUCH_MOVE)
	{
		events::TouchEvent&e = static_cast<events::TouchEvent&>(event);
		emitter->setPosition(e.stageX, e.stageY);
	}
	if (type == events::FILE_SELECTED)
	{
		FileUtil::getInstance().removeEventListener(events::FILE_SELECTED, this);

		events::FileUtilEvent& e = static_cast<events::FileUtilEvent&>(event);

		switch (curretFileOperation)
		{
			case LoadEmitter:
			{
				path = e.file;
				loadEmitter(path.string());
				setTitle(path.filename().string());
			}; break;
			
			case SaveEmitter:
			{
				path = e.file;
				descriptor.save(path.string());
				setTitle(path.filename().string());
			}; break;

			case ChooseImage:
			{
				descriptor.file = e.file;
				updateImageName();
			}; break;
		}
	}
	if (type == events::WIN_EVENT)
	{
		events::WinEvent& e = static_cast<events::WinEvent&>(event);
		if (e.msg == WM_KEYDOWN)
		{
			if (e.wParam == 0x53 && GetKeyState(VK_CONTROL) &&
				TwGetTopBar() == this->bar)
			{
				save();
			}
		}
	}
}

void EmitterEditor::setTitle(const std::string& name)
{
	// Update window title
	std::ostringstream oss;
	oss << this->name << " label='" << name << "' ";
	TwDefine(oss.str().c_str());
}

void EmitterEditor::updateImageName()
{
	std::ostringstream oss;
	oss << this->name << "/choose_image label='" << rag::fs::path(descriptor.file).filename().string() << "' ";
	TwDefine(oss.str().c_str());
}

void EmitterEditor::loadEmitter(std::string file)
{
	path = rag::fs::path(file);

	rag::resourcemgr.clean();
	particles::Descriptor::clearCache();
	descriptor = particles::Descriptor(file);

	colorStart = rag::Color::createABGR(	descriptor.colorStart.r * 255,
									descriptor.colorStart.g * 255,
									descriptor.colorStart.b * 255,
									descriptor.colorStart.a * 255);

	colorEnd = rag::Color::createABGR(	descriptor.colorEnd.r * 255,
									descriptor.colorEnd.g * 255,
									descriptor.colorEnd.b * 255,
									descriptor.colorEnd.a * 255);


	direction[0] = descriptor.velocityNormDir.x;
	direction[1] = -descriptor.velocityNormDir.y;
	direction[2] = 0.0f;

	updateImageName();
}



////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

Editor* Editor::instance = NULL;

Editor::Editor():
fps(0),
numParticles(0),
numFrames(0)
{
	instance = this;

	TwInit(TW_OPENGL, NULL);
	TwWindowSize(rag::Screen::realWidth, rag::Screen::realHeight);

	TwBar *bar = TwNewBar("General");
	TwDefine("General size='240 250'");
	TwDefine("General position='500 0'");

	TwAddVarRO(bar, "fps", TW_TYPE_INT32, &fps, NULL);
	TwAddVarRO(bar, "particles", TW_TYPE_INT32, &numParticles, NULL);

	TwAddSeparator(bar, NULL, NULL);

	TwAddVarRW(bar, "background color", TW_TYPE_COLOR32, &backgroundColor, "coloralpha=false");

	TwAddSeparator(bar, NULL, NULL);

	static ClientData scale_more(this, NULL, "scale+");
	TwAddButton(bar, "scale+", Callback, &scale_more, " label='scale +' ");
	static ClientData scale_less(this, NULL, "scale-");
	TwAddButton(bar, "scale-", Callback, &scale_less, " label='scale -' ");

	TwAddSeparator(bar, NULL, NULL);

	static ClientData add(this, NULL, "add");
	TwAddButton(bar, "add", Callback, &add, " label='add new emitter' ");

	addChild(new EmitterEditor());


	timer.start(0);
}

Editor::~Editor()
{
	TwTerminate();
}

void Editor::logicUpdate()
{
	++numFrames;

	if (timer.finished())
	{
		fps = numFrames;
		numFrames = 0;
		timer.start(1);
	}

	numParticles = 0;
	for (int i = 0; i < childs.size(); i++)
	{
		EmitterEditor* e = dynamic_cast<EmitterEditor*>(childs[i]);
		if (!e) continue;
		numParticles += e->getParticlesCount();
	}
}

void TW_CALL Editor::Callback(void *clientData)
{ 
	if (!clientData)
		return;

	ClientData* data = (ClientData*)(clientData);
	if (data->operation == "add")
	{
		instance->addChild(new EmitterEditor());
	}
	else if (data->operation == "scale+")
	{
		scaleAll(1.1f);
	}
	else if (data->operation == "scale-")
	{
		scaleAll(0.9f);
	}

}

void Editor::render()
{
	DisplayObject::render();
	TwDraw();
}

void Editor::scaleAll(float scale)
{
	for (int i = 0; i < instance->childs.size(); i++)
	{
		EmitterEditor* e = dynamic_cast<EmitterEditor*>(instance->childs[i]);
		if (!e) continue;
		e->descriptor.scale(scale);
	}
}


#endif // _WIN32


