#ifndef _Editor_H_
#define _Editor_H_

#include <rag/DisplayObject.h>
#include "particles/Emitter.h"
#include "particles/Descriptor.h"

#ifdef _WIN32
#include <AntTweakBar.h>
#include <rag/File.h>
#endif

namespace particles
{
	struct ClientData
	{
		ClientData(rag::DisplayObject* editor = NULL, events::EventListener* listener = NULL, std::string operation = ""):
		editor(editor),
		listener(listener),
		operation(operation)
		{
		}

		events::EventListener* listener;
		rag::DisplayObject* editor;
		std::string operation;
	};

	class EmitterEditor: public rag::DisplayObject, events::EventListener
	{

	public:

		EmitterEditor();
		virtual ~EmitterEditor();

		int getParticlesCount() { return particleCount; }

		particles::Descriptor	descriptor;

	private:

		virtual void logicUpdate();
		virtual void onEvent(const std::string& type, events::Event& event);

		void setTitle(const std::string& name);
		void updateImageName();
		void loadEmitter(std::string file);

		void saveas();
		void save();

		static void TW_CALL Callback(void *clientData);

		particles::Emitter*		emitter;
		
		enum FileOperation
		{
			LoadEmitter,
			SaveEmitter,
			ChooseImage,

			None
		};

		static FileOperation curretFileOperation;

		int colorStart;
		int colorEnd;
		glm::vec3 direction;

		int particleCount;

		enum ClientDataType
		{
			CD_Close = 0,
			CD_ChooseImage,
			CD_Save,
			CD_SaveAs,
			CD_Load,
			CD_MoveToTop,

			CD_NumTypes
		};
		ClientData clientData[CD_NumTypes];

		TwBar *bar;

		rag::fs::path path;
		Descriptor::SystemType lastSystemType;

	};

	class Editor: public rag::DisplayObject
	{
	public:

		Editor();
		virtual ~Editor();

		virtual void logicUpdate();
		virtual void render();

	private:

		static void TW_CALL Callback(void *clientData);

		static void scaleAll(float scale);

		static Editor* instance;

		int numParticles;
		int numFrames;
		int fps;
		rag::Timer timer;
	};
}

#endif
