#include "ButtonBehaviour.h"
#include <rag/Platform.h>
//#include "audio/AudioManager.h"
// #include "SimpleAudioEngine.h"

using namespace rag;
using namespace events;

ButtonBehaviour::ButtonBehaviour(std::string soundName, bool buttonForList, bool isAlphaChanging) :
isAlphaChanging(isAlphaChanging),
added(false),
mouseDown(false),
enabled(true),
highlighted(false)
{
    this->soundName = soundName;
    this->buttonForList = buttonForList;
    name = "ButtonBehaviour";
    addEventListener(events::REMOVED, this);
}

void ButtonBehaviour::addListeners()
{
	parent->addEventListener(events::TOUCH_BEGIN, this);
    parent->addEventListener(events::TOUCH_END, this);
    parent->addEventListener(events::TOUCH_MOVE, this);
    parent->addEventListener(events::TOUCH_OUT, this);
    parent->addEventListener(events::TOUCH_TAP, this);
}

void ButtonBehaviour::removeListeners()
{
	parent->removeEventListener(events::TOUCH_BEGIN, this);
    parent->removeEventListener(events::TOUCH_END, this);
    parent->removeEventListener(events::TOUCH_MOVE, this);
    parent->removeEventListener(events::TOUCH_OUT, this);
    parent->removeEventListener(events::TOUCH_TAP, this);
}

void ButtonBehaviour::logicUpdate()
{
    if (!added && parent)
    {
        addListeners();
		//parent->captureInput = true;
		//parent->checkHitPoint = true;
        added = true;
    }
    
    if (timer.finished() && mouseDown)
    {
		glm::vec2 p;
        mouseDownPoint = localToGlobal(p);
        mouseDown = true;

		onStateChanged();
    }

	if (!parent->visible && mouseDown)
	{
		mouseDown = false;

		onStateChanged();
	}
}

void ButtonBehaviour::onEvent(const std::string& type, Event& event)
{
    if (type == events::TOUCH_BEGIN)
    {
		if (!enabled)
			return;
    
        glm::vec2 p;
        mouseDownPoint = localToGlobal(p);
        mouseDown = true;
        
        if (buttonForList)
        {
            timer.start(0.1);
        }
        else
        {
			onStateChanged();
        }
    }
    else if (type == events::TOUCH_END || type == events::TOUCH_OUT)
    {
		if (!enabled)
			return;

        mouseDown = false;

		onStateChanged();
    }
    else if (type == events::TOUCH_MOVE)
    {
		if (!enabled)
			return;

        if (mouseDown)
        {
			glm::vec2 p;
			glm::vec2 myPos = localToGlobal(p);
            if (myPos != mouseDownPoint)
            {
                // Somebody have moved me (a scroll list, maybe?)
                mouseDown = false;

				onStateChanged();
            }
        }
    }
    else if (type == events::TOUCH_TAP)
    {
		if (!enabled)
			return;

        if (!soundName.empty())
        {
            //CocosDenshion::SimpleAudioEngine::sharedEngine()->playEffect(soundName.c_str());
        }
    }
    else if (type == events::REMOVED)
    {
       removeListeners();
    }
}

void ButtonBehaviour::setEnabled(bool value, bool changeAlpha)
{
    if (value == enabled) return;
    this->enabled = value;
	if (changeAlpha)
		onStateChanged();
}

void ButtonBehaviour::setHighlighted(bool value)
{
	if (value==highlighted) return;
	highlighted = value;
	onStateChanged();
}

void ButtonBehaviour::onStateChanged()
{
	if (!isAlphaChanging)
		return;

	if (highlighted && enabled) {
		if (mouseDown)
			parent->color = Color(0.5, 0.5, 0.25);
		else
			parent->color = Color(1, 1, 0.5);
	}
	else {
		if (mouseDown && enabled)
			parent->color = Color(0.5, 0.5, 0.5);
		else
			parent->color = Color(1, 1, 1, enabled ? 1 : 0.25);
	}
}
