#ifndef rmm_Delay_h
#define rmm_Delay_h

#include <rag/DisplayObject.h>
#include <rag/Timer.h>

namespace behaviours
{
    /**
     * Delays the insertion of an object in the display list.
     * Combined with other effects, you can delay them like this:
     *
     * myObject->addChild(new Delay(new AlphaTo(0, 0.5), 3));
     *
     * myObject will start to change its alpha in 3 seconds, 
     * then the alpha out effect will last 0.5 seconds.
     *
     * The object to insert should not start timers in its constructor.
     *
     * DO NOT add the object to delay manually anywhere in display list,
     * let Delay do it.
     *
     */
    class Delay: public rag::DisplayObject
    {
        
    public:
        
        Delay(DisplayObject* object, float delay = 1);
        virtual ~Delay();

		void speedUp();
        
    private:
        
        virtual void logicUpdate();
        
        DisplayObject* object;
        rag::Timer timer;
        
        bool objectAdded;
        bool forceFinish;
    };
}

#endif
