#include "ScaleTo.h"

using namespace behaviours;

ScaleTo::ScaleTo(float target, float time, rag::Ease::EaseType ease):
acquired(false),
origin(0),
time(time),
ease(ease)
{
	this->name = SCALE_ID;
    this->target = target;
}

void ScaleTo::logicUpdate()
{
    if (!parent) return;
    
    if (!acquired)
    {
        origin = parent->getScaleX();
        acquired = true;
		timer.start(time);
    }
    
    float delta = timer.getDelta(ease);
    float scale = origin * (1-delta) + target * delta;
    parent->setScaleX(scale);
    parent->setScaleY(scale);
    
    if (timer.finished())
        parent->deleteChild(this);
}



///////////////////////////////////////////////////////////////////////////////

ScaleFromTo::ScaleFromTo(float origin, float target, float time, rag::Ease::EaseType ease)
{
	this->name = SCALE_ID;
    this->origin = origin;
    this->target = target;
    this->time = time;
    this->ease = ease;
}

void ScaleFromTo::logicUpdate()
{
    if (!parent) return;
    
    if (!timer.running())
        timer.start(time);
    
    float delta = timer.getDelta(ease);
    float scale = origin * (1-delta) + target * delta;
    parent->setScaleX(scale);
    parent->setScaleY(scale);
    
    if (timer.finished())
    {
        events::Event e(events::SCALE_COMPLETE);
        dispatchEvent(e);
        parent->deleteChild(this);
    }
}



///////////////////////////////////////////////////////////////////////////////

ScaleFromToX::ScaleFromToX(float origin, float target, float time, rag::Ease::EaseType ease)
{
	this->name = SCALE_ID;
    this->origin = origin;
    this->target = target;
    this->time = time;
    this->ease = ease;
}

void ScaleFromToX::logicUpdate()
{
    if (!parent) return;
    
    if (!timer.running())
        timer.start(time);
    
    float delta = timer.getDelta(ease);
    float scale = origin * (1-delta) + target * delta;
    parent->setScaleX(scale);
    
    if (timer.finished())
    {
        events::Event e(events::SCALE_COMPLETE);
        dispatchEvent(e);
        parent->deleteChild(this);
    }
}


///////////////////////////////////////////////////////////////////////////////

ScaleFromToY::ScaleFromToY(float origin, float target, float time, rag::Ease::EaseType ease)
{
	this->name = SCALE_ID;
	this->origin = origin;
	this->target = target;
	this->time = time;
	this->ease = ease;
}

void ScaleFromToY::logicUpdate()
{
	if (!parent) return;

	if (!timer.running())
		timer.start(time);

	float delta = timer.getDelta(ease);
	float scale = origin * (1 - delta) + target * delta;
	parent->setScaleY(scale);

	if (timer.finished())
	{
		events::Event e(events::SCALE_COMPLETE);
		dispatchEvent(e);
		parent->deleteChild(this);
	}
}







