#include "Localization.h"
#include <rag/File.h>
#include <map>
#include <vector>
#include <stdarg.h>
#include <sstream>
#include <stdio.h>
#include <string.h>
#include "shared/utils/XML.h"
#include <rag/Platform.h>

#ifdef __APPLE__
    #include "ios/iosUtils.h"
#elif ANDROID
    #include "androidUtils.h"
#endif

using namespace std;



namespace rag
{
    std::map<std::string, std::string> dictionary;    
    std::vector<std::string> split(const std::string& text, std::string delimiter);
	std::string	selectedLang = "en";
	Content* localizeContent = NULL;
    
    string replace(const string& text, const string& replace, const string& replacement)
    {
        string answer(text);
        
        size_t rl = replace.length();
        size_t tl = text.length() - rl;
		for (size_t i = 0; i <= tl && i < text.length(); i++)
        {
            if (text.compare(i, rl, replace) == 0)
            {
                answer = text.substr(0, i);
                answer.append(replacement);
                answer.append(text.substr(i+rl, string::npos));
                return answer;
            }
        }
        
        return answer;
    }

	void parseElements(xmlNode* a_node)
	{
		xmlNode *cur_node = NULL;

		for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
			if (cur_node->type == XML_ELEMENT_NODE) {
			
				if (strcmp((char*)cur_node->name, "row") == 0) {				
					std::string tid = xml::getAsString(cur_node, "tid", "");
                    std::string default_english = xml::getAsString(cur_node, "en", "#" + tid);
					std::string text = xml::getAsString(cur_node, selectedLang.c_str(), default_english);
					dictionary[tid] = text;
				}

			}

			parseElements(cur_node->children);
		}
	}
    
    void load()
    {
        string language = selectedLang;
        
#ifdef __APPLE__
        //language = iosUtils::getLanguage();
#else
        //language = "en";
#endif
        
#ifdef SEPARATED_FILES
        File file("assets/localization/" + language + ".txt");
        bool exists = file.open();
        if (!exists)
        {
            language = "en";
            file = File("assets/localization/" + language + ".txt");
            file.open();
        }
        long filesize = file.getSize();
        char* buffer = (char*)malloc(filesize);
        file.read(buffer, filesize);
        file.close();
        
        std::string s(buffer);
        
        vector<string> lines = split(s, "\n");
        
        int size = lines.size();
        
        for (int i = 0; i < size; i++)
        {
            vector<string> word = split(lines[i], "=");
            
            if (word.size() == 2)
                dictionary[word[0]] = word[1];
        }

        free(buffer);
#else
		if (localizeContent == NULL)
		{
			LOGE("No LocalizeContent found");
			return;
		}

		std::string buffer = localizeContent->load("lockit.xml"); // UtilsHelper::loadContentFile("lockit.xml");

		xmlDoc *doc = NULL;
		xmlNode *root_element = NULL;

		doc = xmlParseMemory(buffer.c_str(), buffer.size());

		xmlErrorPtr errorPtr = xmlGetLastError();
		if (errorPtr)
		{
			LOGE("Error: %s", errorPtr->message);
		}
		else
		{
            dictionary.clear();
			root_element = xmlDocGetRootElement(doc);
			parseElements(root_element);
		}
		xmlFreeDoc(doc);
		
#endif

    }
    
	void localize_reload()
	{
		load();
	}

    std::string localize(const std::string &text_id, 
                         const std::string &p1,
                         const std::string &p2,
                         const std::string &p3,
                         const std::string &p4)
    {
        if (dictionary.size() == 0)
            load();
        
        map<string, string>::iterator it = dictionary.find(text_id);
        
        if (it == dictionary.end())
            return "#" + text_id;
        
        string translation = it->second;
        
        if (!p1.empty()) translation = replace(translation, "%P1", p1);
        if (!p2.empty()) translation = replace(translation, "%P2", p2);
        if (!p3.empty()) translation = replace(translation, "%P3", p3);
        if (!p4.empty()) translation = replace(translation, "%P4", p4);
        
        return translation;
    }
    
    bool localize_exists(const std::string& text_id)
    {
        return dictionary.find(text_id) != dictionary.end();
    }
    
    
    std::vector<std::string> split(const std::string& text, std::string delimiter)
    {
        vector<string> tokens;
        
        // Skip delimiters at beginning.
        std::string::size_type lastPos = text.find_first_not_of(delimiter, 0);
        // Find first "non-delimiter".
        std::string::size_type pos     = text.find_first_of(delimiter, lastPos);
        
        while (std::string::npos != pos || std::string::npos != lastPos)
        {
            // Found a token, add it to the deque.
            tokens.push_back(text.substr(lastPos, pos - lastPos));
            // Skip delimiters.  Note the "not_of"
            lastPos = text.find_first_not_of(delimiter, pos);
            // Find next "non-delimiter"
            pos = text.find_first_of(delimiter, lastPos);
        }
        return tokens;
    }
    
	void setLanguage(std::string newLang)
	{
		if (selectedLang == newLang)
		{
			return;
		}
		selectedLang = newLang;
		if (dictionary.size() != 0)
		{
			load();
		}
	}
    
    
    std::string getDefaultLanguage()
    {
#if __APPLE__
        return iosUtils::getLanguage();
#elif ANDROID
        return AndroidUtils::getLanguage();
#endif
        return "en";
    }

	void setLocalizeContent(Content* content)
	{
		localizeContent = content;
	}
    

} // rag



