#ifndef Rag_TextUtils_h
#define Rag_TextUtils_h

#include <string>
#include <vector>
#include <sstream>

namespace rag
{
    class TextUtils
    {
    public:
        
        /**
         * Splits a string for every delim found.
         */
        static std::vector<std::string> split(const char* s, char delim);
        
        /**
         * Same as above, result is stored @ elems.
         */
        static std::vector<std::string> &split(const char* s, char delim, std::vector<std::string> &elems);
        
        /**
         * Joins the given vector of strings using a separator.
         */
        static std::string join(const std::vector<std::string>& tokens, std::string separator = " ");
        
        /**
         * Splits a text in lines considering a monospaced font.
         */
        static std::vector<std::string> splitLinesMono(const std::vector<std::string>& tokens, int lettersInLine);
        
        static int decodeUTF8(const char *encodedBuffer, unsigned int *outLength);
        static int* decodeText(const char* text, int* size);
        
        /*
         * Splits an UTF8 encoded string and returns a vector with one letter x element.
         */
        static std::vector<std::string> utf8ToVector(const std::string& encodedBuffer, bool removeWhitespace = false);
        static void utf8ToVector(const std::string& encodedBuffer, std::vector<std::string>& output);
        
        static std::string toUpperUTF8(const std::string& s);
        static std::string toLowerUTF8(const std::string& s);
        
		/// Removes all instances of c in s.
		static std::string remove(const std::string& s, char c);
        static std::string trimWhiteSpace(const std::string& s, bool begin = false, bool end = false);
        
        /*
         * Returns a text with first letter capitalized. Supports UTF8.
         */
        static std::string capitalizeFirst(const std::string& s);
        
        /*
         * For n < 100,000 returns the same number
         * For 1,000,000 > n >= 100,000 returns (n/1000)K       (i.e., 125K)
         * For n > 1,000,000 returns (n/1,000,000),(n%100,000)M (i.e., 2,3M)
         */
        static std::string bigNumberFormat(int n);
		static std::string bigNumberAddCommas(int n);
    };
}

#endif
