#include "TextView.h"
#include <rag/File.h>

using namespace rag;

TextView::TextView() :
isOpen(false)
{
}

TextView::~TextView()
{
	close();
}

void TextView::clear()
{
	content.clear();
}

void TextView::add(const std::string& value)
{
	content.append(value);
}

void TextView::loadFromFile(const std::string& filename)
{
	content = File::load(filename, false, "rb", false);
}

#ifdef _WIN32
#define IDC_MAIN_EDIT	101
#endif

void TextView::open()
{
	if (isOpen) return;
	isOpen = true;

	rag::Rectangle rect = getBounds();
#ifdef _WIN32
	// Create an edit box
	HWND hWnd = (HWND)winutils.getWindow();
	HWND hEdit = CreateWindowEx(WS_EX_CLIENTEDGE,
								"EDIT",
								"",
								WS_CHILD|WS_VISIBLE|
								ES_READONLY|ES_MULTILINE|ES_AUTOVSCROLL|ES_AUTOHSCROLL,
								int(rect.x),
								int(rect.y),
								int(rect.width),
								int(rect.height),
								hWnd,
								(HMENU)IDC_MAIN_EDIT,
								GetModuleHandle(NULL),
								NULL);

	HGDIOBJ hfDefault=GetStockObject(DEFAULT_GUI_FONT);
	SendMessage(hEdit, WM_SETFONT, (WPARAM)hfDefault, MAKELPARAM(FALSE,0));

	SendMessage(hEdit, WM_SETTEXT, NULL, (LPARAM)content.c_str());

	int min, max;
	if (GetScrollRange(hEdit, SB_VERT, &min, &max))
		SetScrollPos(hEdit, SB_VERT, max, TRUE);

	handle = (void*)hEdit;
#endif
}

void TextView::close()
{
	if (!isOpen) return;
	isOpen = false;

#ifdef _WIN32
	DestroyWindow((HWND)handle);
#endif
}
