#include "TimeUtils.h"
#include "Localization.h"
#include <vector>
#include <math.h>
#include <iostream>
#include <sstream>
#include <time.h>
#include <cmath>
#include <algorithm>
#include <chrono>
#include <iomanip>
#include <ctime>

using namespace rag;
using namespace std;


/*
 TID_GEN_TIME_SECONDS=seconds
 TID_GEN_TIME_MINUTES=minutes
 TID_GEN_TIME_HOURS=hours
 TID_GEN_TIME_HOUR=hour
 TID_GEN_TIME_DAYS=days
 TID_GEN_TIME_DAY=day
 TID_GEN_TIME_SECONDS_ABR=sec.
 TID_GEN_TIME_MINUTES_ABR=min.
 TID_GEN_TIME_HOURS_ABR=h.
 TID_GEN_TIME_DAYS_ABR=d.
*/

#define SC_IN_WEEK 604800
#define SC_IN_DAY 86400
#define SC_IN_HOUR 3600
#define SC_IN_MINUTE 60


string TimeUtils::formatTime(float _seconds, bool withSeconds)
{
	char buffer[128];
	_seconds = std::ceil(_seconds);
	if(_seconds > SC_IN_WEEK)
	{
		int days = _seconds / SC_IN_DAY;
		int weeks = _seconds / SC_IN_WEEK;

		days -= (weeks * SC_IN_WEEK ) / SC_IN_DAY;

		if(weeks > 0 && days == 0)
		{
			int hours = (_seconds - (weeks * SC_IN_WEEK )) / SC_IN_HOUR;
			sprintf(buffer, "%01dw %01dh", weeks, hours);
		}
		else if(weeks > 0)
			sprintf(buffer, "%01dw %01dd", weeks, days);
		else
			sprintf(buffer, "%01dw", weeks);
	}
	else if(_seconds > SC_IN_DAY)
	{
		int hours = _seconds / SC_IN_HOUR;
		int days = _seconds / SC_IN_DAY;
		hours -= (days * SC_IN_DAY ) / SC_IN_HOUR;

		if(hours > 0)
			sprintf(buffer, "%01dd %01dh", days, hours);
		else
			sprintf(buffer, "%01dd", days);
	}
	else if(_seconds > SC_IN_HOUR)
	{
		int minutes = _seconds / SC_IN_MINUTE;
		int hours = _seconds / SC_IN_HOUR;
		minutes -= (hours * SC_IN_HOUR ) / SC_IN_MINUTE;

		if(minutes > 0)
			sprintf(buffer, "%01dh %01dm", hours, minutes);
		else 
			sprintf(buffer, "%01dh", hours);
	}
	else if(_seconds > SC_IN_MINUTE)
	{
		int seconds = (int)(_seconds);
    
		int minutes = seconds / SC_IN_MINUTE;
		seconds -= minutes*SC_IN_MINUTE;
		if(withSeconds && seconds > 0)
			sprintf(buffer, "%01dm %01ds", minutes, seconds);
		else
			sprintf(buffer, "%01dm", minutes);
	}
	else if (!withSeconds)
	{
		sprintf(buffer, "1 m");
	}
	else
	{
		int seconds = (int)(_seconds);
		sprintf(buffer, " %01ds", seconds);
	}
    
    
    
    
    return buffer;
}

string TimeUtils::format(float _seconds, int numFields)
{
    if (numFields < 1) return "";
    
	int seconds = (int)(std::ceil(_seconds));
    
    int days = seconds / SC_IN_DAY;
    seconds -= days * SC_IN_DAY;
    
    int hours = seconds / SC_IN_HOUR;
    seconds -= hours*SC_IN_HOUR;
    
    int minutes = seconds / SC_IN_MINUTE;
    seconds -= minutes*SC_IN_MINUTE;
    
    vector<int> v;
    v.push_back(days);
    v.push_back(hours);
    v.push_back(minutes);
    v.push_back(seconds);
    
    const char* tids[] = 
    {
        "TID_GEN_TIME_DAYS_ABR",
        "TID_GEN_TIME_HOURS_ABR",
        "TID_GEN_TIME_MINUTES_ABR",
        "TID_GEN_TIME_SECONDS_ABR"
    };
    
    // find first value
    int first = -1;
    
    for (int i = 0; i < v.size(); i++)
        if (v[i] != 0)
        {
            first = i;
            break;
        }
    
    if (first == -1) return "";
    
    int last = std::min((int)first + numFields, (int)v.size());
    
    ostringstream oss;
    
    for (int i = first; i < last; i++)
    {   
        oss << v[i] << localize(tids[i]) << " ";
    }
    
    return oss.str();
}



string TimeUtils::formatHMS(float _seconds)
{
    int seconds = (int)(_seconds + 0.5);
    
    
    int hours = seconds / SC_IN_HOUR;
    seconds -= hours*SC_IN_HOUR;
    
    int minutes = seconds / SC_IN_MINUTE;
    seconds -= minutes*SC_IN_MINUTE;
    
    char buffer[128];
    
    sprintf(buffer, "%02d:%02d:%02d", hours, minutes, seconds);
    
    return buffer;
}

string TimeUtils::formatMS(float _seconds)
{
    int seconds = (int)(_seconds + 0.5);
    
    int minutes = seconds / SC_IN_MINUTE;
    seconds -= minutes*SC_IN_MINUTE;
    
    char buffer[128];
    
    sprintf(buffer, "%02d:%02d", minutes, seconds);
    
    return buffer;
}

string TimeUtils::getElapsedTime(time_t from)
{
    
    time_t t1 = from;
    time_t t2 = time(NULL);
    double diff = difftime(t2, t1);
    
    int months = diff / 3600 / 24 / 30;
    diff -= months * 3600 * 24 * 30;
    
    int days = diff / 3600 / 24;
    diff -= days * 3600 * 24;
    
    int hours = diff / 3600;
    diff -= hours * 3600;
    
    int minutes = diff / 60;
    diff -= minutes * 60;
    
    int seconds = diff;
    
    ostringstream oss;
    
    if (months > 0)
    {
        oss << months;
        if (months == 1) return localize("TID_RMM_TIME_AGO", oss.str(), localize("TID_GEN_TIME_MONTH"));
        else return localize("TID_RMM_TIME_AGO", oss.str(), localize("TID_GEN_TIME_MONTHS"));
    }
    else if (days > 0)
    {
        oss << days;
        if (days == 1) return localize("TID_RMM_TIME_AGO", oss.str(), localize("TID_GEN_TIME_DAY"));
        else return localize("TID_RMM_TIME_AGO", oss.str(), localize("TID_GEN_TIME_DAYS"));
    }
    else if (hours > 0)
    {
        oss << hours;
        if (hours == 1) return localize("TID_RMM_TIME_AGO", oss.str(), localize("TID_GEN_TIME_HOUR"));
        else return localize("TID_RMM_TIME_AGO", oss.str(), localize("TID_GEN_TIME_HOURS"));
    }
    else if (minutes > 0)
    {
        oss << minutes;
        if (minutes == 1) return localize("TID_RMM_TIME_AGO", oss.str(), localize("TID_GEN_TIME_MINUTE"));
        else return localize("TID_RMM_TIME_AGO", oss.str(), localize("TID_GEN_TIME_MINUTES"));
    }
    else
    {
        //return "moments ago"; //TEMP
        oss << seconds;
        if (seconds == 1) return localize("TID_RMM_TIME_AGO", oss.str(), localize("TID_GEN_TIME_SECOND"));
        else return localize("TID_RMM_TIME_AGO", oss.str(), localize("TID_GEN_TIME_SECONDS"));
    }
    return "";
}

std::string TimeUtils::getDateString()
{
	time_t now = time(0);
	return getDateString(now);
}

std::string TimeUtils::getDateString(time_t date)
{
	ostringstream oss;

	struct tm * now = localtime(&date);
	oss << (now->tm_year + 1900) << "-"
		<< (now->tm_mon + 1) << "-"
		<< now->tm_mday;

	return oss.str();
}

std::string TimeUtils::getTimeString(time_t datetime)
{
	ostringstream oss;

	struct tm * now = localtime(&datetime);
	oss << (now->tm_year + 1900) << "-"
		<< (now->tm_mon + 1) << "-"
		<< now->tm_mday << " "
		<< now->tm_hour << ":"
		<< now->tm_min << ":"
		<< now->tm_sec;

	return oss.str();
}

std::string TimeUtils::now(std::string format)
{
#ifndef __linux__
	auto now = std::chrono::system_clock::now();
	auto in_time_t = std::chrono::system_clock::to_time_t(now);

	std::stringstream ss;
	ss << std::put_time(std::localtime(&in_time_t), format.c_str());
	return ss.str();
#else
	// std::put_time not implemented yet in GCC 4.8.2
	time_t date = time(0);
	struct tm * now = localtime(&date);
	std::stringstream ss;
	ss << (now->tm_year + 1900) << "-"
		<< (now->tm_mon + 1) << "-"
		<< now->tm_mday << ' '
		<< now->tm_hour << ':'
		<< now->tm_min << ':'
		<< now->tm_sec;
	return ss.str();
#endif
}