#ifdef WIN32

#include "Test.h"
#include <rag/Bitmap.h>
#include <rag/Screen.h>
#include "win/winUtils.h"
#include <rag/XFLParser.h>
#include <rag/rag.h>

namespace test
{
class RotatingLogo : public rag::DisplayObject
{
public:
	RotatingLogo()
	{
		rag::Renderer::getInstance().setClearColor(rag::Color::white * 0.4f);
		logo = new rag::DisplayObject();
		auto bmp = new rag::Bitmap("assets/textures/logo_512.png");
		bmp->setPosition(-bmp->getWidth() / 2, -bmp->getHeight() / 2);
		logo->addChild(bmp);
		logo->setPosition(rag::Screen::center);
		addChild(logo);
	}

	virtual void logicUpdate() override
	{
		logo->setScaleX(std::sin(rag::Timer::totalTime));
	}

	rag::DisplayObject* logo;
};

class BurnUniformSinus : public rag::DisplayObject
{
public:

	BurnUniformSinus():
		value(.0f)
	{
	}

	virtual void logicUpdate() override
	{
		value += rag::Timer::deltaTime * 4.0f;
		parent->material.setUniform("uBurnFactor", std::abs(std::sin(value)));
	}

	float value;
};

class MultipleMaterial : public rag::DisplayObject
{
public:

	MultipleMaterial()
	{
		for (int i = 0; i < 3; i++)
		{
			rag::Bitmap* bmp = new rag::Bitmap("assets/arch_tower_09.png");
			bmp->setPosition(rag::Screen::width / 5 * (i + 1), rag::Screen::center.y);
			addChild(bmp);

			if (i == 0)
			{
				bmp->material = rag::Material("assets/shaders/Tint.vsh", "assets/shaders/Desaturate.fsh");
			}

			if (i == 1)
			{
				bmp->material.fsh = "assets/shaders/Burn.fsh";
				bmp->material.setUniform("uBurnFactor", 1.0);
				bmp->addChild(new BurnUniformSinus());
			}
		}
	}

};

class MultipleMaterialWithPrority : public rag::DisplayObject
{
public:

	MultipleMaterialWithPrority()
	{
		for (int i = 0; i < 3; i++)
		{
			auto container = new rag::DisplayObject();
			rag::Bitmap* bmp = new rag::Bitmap("assets/arch_tower_09.png");
			container->addChild(bmp);
			container->setPosition(rag::Screen::width / 5 * (i + 1), rag::Screen::center.y);
			addChild(container);

			if (i == 0)
			{
				container->material = rag::Material("assets/shaders/Tint.vsh", "assets/shaders/Desaturate.fsh");
			}

			if (i == 1)
			{
				container->material.fsh = "assets/shaders/Burn.fsh";
				container->material.setUniform("uBurnFactor", 1.0);
				container->addChild(new BurnUniformSinus());
			}

			container->material.priority = 1; // propagate priority
		}
	}

};

class ClippingRectangle : public rag::DisplayObject
{
public:

	ClippingRectangle()
	{
		for (int i = 0; i < 3; i++)
		{
			auto container = new rag::DisplayObject();
			rag::Bitmap* bmp = new rag::Bitmap("assets/arch_tower_09.png");
			container->addChild(bmp);
			container->setPosition(rag::Screen::width / 5 * (i + 1), rag::Screen::center.y);
			addChild(container);

			if (i == 0)
			{
				container->material = rag::Material("assets/shaders/Tint.vsh", "assets/shaders/Desaturate.fsh");
			}

			if (i == 1)
			{
				container->material.fsh = "assets/shaders/Burn.fsh";
				container->material.setUniform("uBurnFactor", 1.0);
				container->addChild(new BurnUniformSinus());
			}

			container->material.priority = 1; // propagate priority
		}
	}

	virtual void logicUpdate() override
	{
		static float a = 0;
		a += rag::Timer::deltaTime;
		rag::Rectangle r;
		r.width = rag::Screen::width * std::abs(std::sin(a));
		r.height = rag::Screen::height;
		this->setClipRectangle(r);
	}

};


class XFLParseAtlas : public rag::DisplayObject
{
public:
	XFLParseAtlas()
	{
		{
			rag::XFLParser p;
			rag::DisplayObject* wall = p.load("assets/flashes/decorations_original/LIBRARY", "wall06.xml");
			wall->setPosition(rag::Screen::center.x - wall->getWidth(), rag::Screen::center.y);
			addChild(wall);
		}
		{
			rag::XFLParser p;
			rag::DisplayObject* wall = p.load("assets/flashes/decorations_atlas/LIBRARY", "wall06.xml");
			wall->setPosition(rag::Screen::center.x + wall->getWidth(), rag::Screen::center.y);
			addChild(wall);
		}
	}
};

} // test


namespace rag
{

Test::Test():
testContainer(nullptr)
{
	WinUtils::getInstance().addEventListener(events::WIN_EVENT, this);

	startNextTest();
}

Test::~Test()
{
	WinUtils::getInstance().removeEventListener(events::WIN_EVENT, this);
}

void Test::startNextTest()
{
	if (testContainer == nullptr)
	{
		testContainer = new rag::DisplayObject();
		addChild(testContainer);
	}

	while (!testContainer->childs.empty())
	{
		testContainer->deleteChild(testContainer->getChildAt(0));
	}

	static int i = 0;
	switch (i)
	{
	case 0: testContainer->addChild(new test::RotatingLogo()); break;
	case 1: testContainer->addChild(new test::MultipleMaterial()); break;
	case 2: testContainer->addChild(new test::MultipleMaterialWithPrority()); break;
	case 3: testContainer->addChild(new test::ClippingRectangle()); break;
	case 4: testContainer->addChild(new test::XFLParseAtlas()); break;
	default:
		i = 0; 
		startNextTest();
		return;
	}
	++i;
}

void Test::onEvent(const std::string& type, events::Event& e)
{
	if (type == events::WIN_EVENT)
	{
		events::WinEvent& event = static_cast<events::WinEvent&>(e);

		switch (event.msg)
		{
		case WM_KEYDOWN:
		{
			onKeyDown(event);
		}; break;
		}
	}
}

void Test::onKeyDown(events::WinEvent& event)
{
	switch (event.wParam)
	{
//	case VK_F5:
//	{
//	}; break;
//
//	case 'Z':
//	{
//	}; break;
//
//	case 'Y':
//	{
//	}; break;

	case VK_RETURN:
	{
		// next test.
		startNextTest();
	}; break;
	}
}

} // rag
#endif