//
//  TestDisplayObject.cpp
//
//
//  Created by Ivan Sanchez Luque on 6/11/13.
//  Copyright (c) 2013 Ivan Sanchez Luque. All rights reserved.
//

#include <rag/DisplayObject.h>

using namespace rag;

int test_initial_values()
{
    int error = 0;
    
    DisplayObject d;
    
    error += d.name.empty() ? 0 : 1;
    error += d.getX() == 0 ? 0 : 1;
    error += d.getY() == 0 ? 0 : 1;
    error += d.getWidth() == 0 ? 0 : 1;
    error += d.getHeight() == 0 ? 0 : 1;
    error += d.getScaleX() == 1 ? 0 : 1;
    error += d.getScaleY() == 1 ? 0 : 1;
    
    return error;
}

int test_add_child()
{
    int error = 0;
    
    DisplayObject d1, d2;
    DisplayObject c1, c2;
    
    d2.addChild(&d1);
    d1.addChild(&c1);
    d1.addChild(&c2);
    
    error += d1.getNumChilds() == 2 ? 0 : 1;
    error += d2.getNumChilds() == 1 ? 0 : 1;
    error += c1.getNumChilds() == 0 ? 0 : 1;
    error += c2.getNumChilds() == 0 ? 0 : 1;
    
    return error;
}

int test_add_child_at()
{
    int error = 0;
    
    DisplayObject a, b, c;
    
    a.addChild(&b);
    a.addChildAt(&c, 0);
    
    error += a.getChildAt(0) == &c ? 0 : 1;
    error += a.getChildAt(1) == &b ? 0 : 1;
    
    return error;
}

int test_contains()
{
    int error = 0;
    
    DisplayObject a, b, c, d;
    
    a.addChild(&b);
    b.addChild(&c);
    
    error += a.contains(&a) ? 0 : 1;
    error += a.contains(&b) ? 0 : 1;
    error += a.contains(&c) ? 0 : 1;
    error += !a.contains(&d) ? 0 : 1;
    
    return error;
}

int test_get_child_by_name()
{
    int error = 0;

    DisplayObject a, b, c;
    
    b.name = "b";
    c.name = "c";
    
    a.addChild(&b);
    a.addChild(&c);
    
    DisplayObject* target = a.getChildByName("b");
    
    error += a.getChildIndex(target) == 0 ? 0 : 1;
    
    return error;
}

int test_swap_children()
{
    int error = 0;
    
    DisplayObject a, b, c;
    b.name = "b";
    c.name = "c";
    
    a.addChild(&b);
    a.addChild(&c);
    
    error += a.getChildAt(0)->name == "b" ? 0 : 1;
    error += a.getChildAt(1)->name == "c" ? 0 : 1;
    
    a.swapChildren(&b, &c);
    
    error += a.getChildAt(0)->name == "c" ? 0 : 1;
    error += a.getChildAt(1)->name == "b" ? 0 : 1;
    
    return error;
}

int test_remove_child()
{
    int error = 0;
    
    DisplayObject a, b, c;
    
    b.name = "b";
    c.name = "c";
    
    a.addChild(&b);
    a.addChild(&c);
    
    error += a.getNumChilds() == 2 ? 0 : 1;
    a.removeChild(&b);
    error += a.getNumChilds() == 1 ? 0 : 1;
    error += a.getChildAt(0)->name == "c" ? 0 : 1;
    
    return error;
}

int test_delete_child()
{
    int error = 0;
    
    DisplayObject a, c;
    DisplayObject* b = new DisplayObject();
    
    b->name = "b";
    c.name = "c";
    
    a.addChild(b);
    a.addChild(&c);
    
    error += a.getNumChilds() == 2 ? 0 : 1;
    a.deleteChild(b);
    error += a.getNumChilds() == 1 ? 0 : 1;
    error += a.getChildAt(0)->name == "c" ? 0 : 1;
    
    DisplayObject::deletePendentObjects();
    
    return error;
}

int test_global_to_local()
{
    int error = 0;
    
    DisplayObject container;
    
    DisplayObject circle;
    circle.setWidth(40);
    circle.setHeight(40);
    circle.setX(10);
    container.addChild(&circle);
    
    glm::vec2 point1(0, 0);
    error += !circle.hitTestPoint(point1.x, point1.y) ? 0 : 1;
	glm::vec2 plocal = circle.globalToLocal(point1);
    error += plocal.x == -10 ? 0 : 1;
    error += plocal.y == 0 ? 0 : 1;
    
	glm::vec2 point2(10, 1);
    error += circle.hitTestPoint(point2.x, point2.y) ? 0 : 1;
    plocal = circle.globalToLocal(point2);
    error += plocal.x == 0 ? 0 : 1;
    error += plocal.y == 1 ? 0 : 1;
    
	glm::vec2 point3(30, 20);
    error += circle.hitTestPoint(point3.x, point3.y) ? 0 : 1;
    plocal = circle.globalToLocal(point3);
    error += plocal.x == 20 ? 0 : 1;
    error += plocal.y == 20 ? 0 : 1;
    
    return error;
}

int test_local_to_global()
{
    int error = 0;
    
    DisplayObject container;
    
    DisplayObject circle;
    circle.setWidth(100);
    circle.setHeight(100);
    circle.setX(100);
    circle.setY(200);
    container.addChild(&circle);
    
	glm::vec2 p1(0, 0);
	glm::vec2 pglobal = circle.localToGlobal(p1);
    
    error += pglobal.x == 100 ? 0 : 1;
    error += pglobal.y == 200 ? 0 : 1;
    
    return error;
}

int test_get_bounds()
{
    int error = 0;
    
    DisplayObject object;
    
    DisplayObject container, contents;
    container.setX(100);
    container.setY(100);
    object.addChild(&container);
    
    DisplayObject circle;
    circle.setWidth(200);
    circle.setHeight(200);
    circle.setX(-100);
    circle.setY(-100);
    contents.addChild(&circle);
    container.addChild(&contents);
    
    rag::Rectangle r = contents.getBounds(&container);
    error += r.x == -100 ? 0 : 1;
    error += r.y == -100 ? 0 : 1;
    error += r.width == 200 ? 0 : 1;
    error += r.height == 200 ? 0 : 1;
    
    r = contents.getBounds(&object);
    error += r.x == 0 ? 0 : 1;
    error += r.y == 0 ? 0 : 1;
    error += r.width == 200 ? 0 : 1;
    error += r.height == 200 ? 0 : 1;
    
    return error;
}

int run_tests()
{
    int error = 0;
    
    error += test_initial_values();
    error += test_add_child();
    error += test_add_child_at();
    error += test_contains();
    error += test_get_child_by_name();
    error += test_swap_children();
    error += test_remove_child();
    error += test_delete_child();
    error += test_global_to_local();
    error += test_local_to_global();
    error += test_get_bounds();
    
    return error;
}