uniform mat4 MVP;
uniform mat4 boneMatrix[50];
uniform mat3 normal_matrix;		// normal matrix
uniform vec3 ec_light_dir; 		// light direction in eye coords

attribute vec3 Position;
attribute vec2 TexCoordinate;	// Per-vertex texture coordinate information we will pass in.
attribute vec4 weight;
attribute vec4 matrixIndex;
attribute vec3 Normal;

varying vec2 v_TexCoordinate;	// This will be passed into the fragment shader.
varying float v_diffuse;


void main(void) {
    v_TexCoordinate = TexCoordinate;
 
 	// Animation transform
    mat4 matTransform = boneMatrix[int(matrixIndex.x)] * weight.x;
    matTransform += boneMatrix[int(matrixIndex.y)] * weight.y;
    matTransform += boneMatrix[int(matrixIndex.z)] * weight.z;
    float finalWeight = 1.0 - ( weight.x + weight.y + weight.z );
    matTransform += boneMatrix[int(matrixIndex.w)] * finalWeight;
    
    // put vertex normal into eye coords
 	vec3 ec_normal = normalize(normal_matrix * Normal);
 	vec4 n = matTransform * vec4(ec_normal, 1.0);
 	// emit diffuse scale factor, texcoord, and position
 	v_diffuse = max(dot(ec_light_dir, n.xyz), 0.0);
 	
    gl_Position = (MVP) * (matTransform * vec4(Position, 1.0));
}