import os
import sys
import time
from xml.dom import minidom
import subprocess
import shutil
import csv

assets_folder = './assets'
atlases_folder = './assets/atlases'

docs = {} # dictionary of parsed xmls
atlases = {} # dictionary of arrays of images. map<atlas, vector<image> > where atlas and image are file names.
library = "" # current library parsed.

def add_symbols_to_atlas(doc, atlas):
    if atlas not in atlases.keys():
            atlases[atlas] = set()
    symbols = doc.getElementsByTagName('DOMSymbolInstance')
    for s in symbols:
        path = os.path.join(library, s.attributes['libraryItemName'].value + '.xml')
        add_symbols_to_atlas(minidom.parse(path), atlas)
    add_images_to_atlas(doc, atlas)

def add_images_to_atlas(doc, atlas):
    bitmaps = doc.getElementsByTagName('DOMBitmapInstance')
    for b in bitmaps:
        atlases[atlas].add(b.attributes['libraryItemName'].value)

def parse_cdata(data):
    data = '<doc>' + data + '</doc>'
    doc = minidom.parseString(data)
    elements = doc.getElementsByTagName('Inspectable')
    for element in elements:
        a = element.attributes['name']
        if a.value == 'atlas':
            return element.attributes['defaultValue'].value
    return 0

def parsefile(file):
    sys.stdout.write('.')
    doc = minidom.parse(file)
    docs[file] = doc
    cdata = doc.getElementsByTagName('parametersAsXML')
    if len(cdata) > 0:
        atlas = parse_cdata(cdata[0].firstChild.wholeText)
        if atlas:
            add_symbols_to_atlas(doc, atlas)

def parselib():
    for dirpath, dirnames, filenames in os.walk(library):
        for filename in [f for f in filenames if f.endswith(".xml")]:
            file = os.path.join(dirpath, filename)
            parsefile(os.path.normpath(file))
    print ''

def generate_atlas():
    for key in atlases:
        l = list()
        for image in atlases[key]:
            l.append(os.path.join(library, image) + ' ')
        program = '../rag/tools/TextureAtlas.exe'
        output = atlases_folder + '/' + key
        result = subprocess.call([program, '-i csv', '-x 2048', '-y 2048', '-q', '-o ' + output, l])
        if result != 0:
            exit()
        parse_csv(output + '.csv')

def splitlibrary(path):
    # Returns a list with exactly 2 strings with library and symbol path
    p = os.path.normpath(path).split(os.sep)
    i = p.index("LIBRARY")
    ret = []
    ret.append( '/'.join(p[0:i+1]) )
    ret.append( '/'.join(p[i+1:]) )
    return ret
    
def parse_csv(file):
    with open(file) as csvfile:
        reader = csv.reader(csvfile, delimiter=',')
        first = 1
        for row in reader:
            if first:
                first = 0
            else:
                parse_csv_row(row)
    return

def parse_csv_row(row):
    atlas = row[0]
    atlas_width = float(row[3])
    atlas_height = float(row[4])
    tex_name = splitlibrary(row[5])[1]
    tex_rotated = int(row[6])
    tex_width = float(row[7])
    tex_height = float(row[8])
    tex_x = float(row[9])
    tex_y = float(row[10])
    
    for f in docs.keys():
        doc = docs[f]
        bitmaps = doc.getElementsByTagName('DOMBitmapInstance')
        for b in bitmaps:
            if b.attributes['libraryItemName'].value == tex_name:
                uvx0 = str(tex_x / atlas_width)
                uvx1 = str((tex_x + tex_width) / atlas_width)
                if tex_rotated:
                    uvx0, uvx1 = uvx1, uvx0
                b.setAttribute('libraryItemName', atlas)
                b.setAttribute('uvx0', uvx0)
                b.setAttribute('uvx1', uvx1)
                b.setAttribute('uvy0', str(tex_y / atlas_height))
                b.setAttribute('uvy1', str((tex_y + tex_height) / atlas_height))
                fw = open(f, 'w')
                fw.write(doc.toxml())
                fw.close()

start = time.time()

if not os.path.exists(assets_folder):
    print 'path', assets_folder, 'not found. Run this script from project root.'
    exit()
    
if not os.path.exists(atlases_folder):    
    os.makedirs(atlases_folder)

# Fill atlases
sys.stdout.write('parsing')
for dirname, dirnames, filenames in os.walk(assets_folder):
    for d in dirnames:
        fulldir = os.path.join(dirname, d)
        library = os.path.join(fulldir, 'LIBRARY')
                
        if os.path.isdir(library):
            parselib()
            generate_atlas()
            docs = {}
            atlases = {}

print "\n%.2f" % (time.time() - start), "seconds"