import os
import sys
import time
from xml.dom import minidom

template_h_start = """
#ifndef FLASH_H_
#define FLASH_H_

#include <rag/DisplayObject.h>
#include <rag/TextField.h>
"""

template_h_end = """
#endif
"""

template_class_dec = """
    class {0} : public rag::DisplayObject
    {{
        public:
        
        static {0}* create();
"""

template_create_imp = """
    {0}* {0}::create()
    {{
        rag::XFLParser p;
        return static_cast<{0}*>(p.load("{1}", "{2}"));
    }}
"""

template_symbol_get = """
    {0}{1}::{2}()
    {{
        return static_cast<{0}>(getChildByName("{2}"));
    }}
"""

nsdic = {} # dictionary of namespaces <namespace, files>
classes = {} # <namespace, classes>
docs = {} # dictionary of parsed xmls

h = open("src/flash.h", 'w')
h.write(template_h_start)
cpp = open("src/flash.cpp", 'w')
cpp.write('#include "flash.h"\n')
cpp.write('#include <rag/XFLParser.h>\n\n')

def namespace(path):
    # Returns a list with the flash namespace after decompose the path
    p = os.path.normpath(path).split(os.sep)
    p.remove("LIBRARY")
    p.pop(0)
    p.pop(-1)
    return p
    
def library_namespace(path):
    # Returns the namespace for the root of a library.
    p = os.path.normpath(path).split(os.sep)
    a = ""
    while a != "LIBRARY":
        a = p.pop(-1)
    p.pop(0)
    return p

def splitlibrary(path):
    # Returns a list with exactly 2 strings with library and symbol path
    p = os.path.normpath(path).split(os.sep)
    i = p.index("LIBRARY")
    ret = []
    ret.append( '/'.join(p[0:i+1]) )
    ret.append( '/'.join(p[i+1:]) )
    return ret
    
def write_opennamespace(file, ns):
    for n in ns:
        file.write( "namespace " + n + "\n{\n")

def class_return_declaration(root_namespace, str):
    # with libraryItemName returns class name
    cns = str.split('/')
    innerclass = cns.pop(-1)
    cns = root_namespace + cns # inner class full namespace
    ns = '/'.join(cns)
    if not ns in classes:
        return "rag::DisplayObject* "
    if innerclass not in classes[ns]:
        return "rag::DisplayObject* "
    else:
        return "::".join(cns) + "::" + innerclass + "* "

def codify_file(path):
    doc = docs[path]
    symbols = [i for i in doc.getElementsByTagName("DOMSymbolInstance") if "name" in i.attributes.keys()]    
    texts = [i for i in doc.getElementsByTagName("DOMDynamicText") if "name" in i.attributes.keys()]
    ns = library_namespace(path)
        
    #declare class
    clazz = os.path.normpath(path).split(os.sep).pop(-1).split('.')[0]
    h.write(template_class_dec.format(clazz))
    
    #declare methods    
    methods = [] # allow duplicated names in flash
    for s in symbols:
        method = s.attributes["name"].value
        if method in methods:
            continue
        methods.append(method)
        cl = class_return_declaration(ns, s.attributes["libraryItemName"].value)
        h.write( "\t\t" + cl + s.attributes["name"].value + "();\n")

    methods = [] # allow duplicated names in flash
    for t in texts:
        method = t.attributes["name"].value
        if method in methods:
            continue
        methods.append(method)
        h.write( "\t\trag::TextField* " + t.attributes["name"].value + "();\n")
            
    # close class
    h.write( "\t};\n" )
    
    # "create" method implementation
    p = splitlibrary(path)
    cpp.write(template_create_imp.format(clazz, p[0], p[1]))
    
    # symbol methods implementation
    methods = [] # allow duplicated names in flash
    for s in symbols:
        method = s.attributes["name"].value
        if method in methods:
            continue
        methods.append(method)
        cl = class_return_declaration(ns, s.attributes["libraryItemName"].value)
        cpp.write(template_symbol_get.format(cl, clazz, method))
            
    methods = [] # allow duplicated names in flash
    for t in texts:
        method = t.attributes["name"].value
        if method in methods:
            continue
        methods.append(method)
        cpp.write(template_symbol_get.format('rag::TextField* ', clazz, method))
    
def read_namespace(path):
    ns = '/'.join(namespace(path))
    if ns not in nsdic:
        nsdic[ns] = []
    nsdic[ns].append(path)
        
        
def is_file_required(doc):
    symbols = doc.getElementsByTagName("DOMSymbolInstance")
    for s in symbols:
        if "name" in s.attributes.keys():
            return 1
            
    texts = doc.getElementsByTagName("DOMDynamicText")
    for t in texts:
        if "name" in t.attributes.keys():
            return 1
    return 0
    
def parsexml(ns, files):
    # find classes to parse (discard classes that doesn't contains names)
    sys.stdout.write( '::'.join(ns) )    
    for f in files:
        sys.stdout.write('.')
        doc = minidom.parse(f)        
        if is_file_required(doc):
            docs[f] = doc
    print

    
def forward_declare_namespace(key):
    list = [i for i in nsdic[key] if i in docs.keys()]
    
    if len(list) == 0:
        return # empty namespace
        
    ns = namespace(nsdic[key][0])
    write_opennamespace(h, ns)
    
    if key not in classes:
        classes[key] = []
    
    for f in sorted(list):
        clazz = os.path.normpath(f).split(os.sep).pop(-1).split('.')[0]
        classes[key].append(clazz)
        h.write( "\tclass " + clazz + ";\n")
    
    h.write("\n}" * len(ns) + "\n");
    
def codify_namespace(key):
    # opens a namespace, codifies files and closes namespace    
    list = [i for i in nsdic[key] if i in docs.keys()]
    
    if len(list) == 0:
        return # empty namespace
    
    ns = namespace(nsdic[key][0])    
    write_opennamespace(h, ns)
    write_opennamespace(cpp, ns)
    
    for f in sorted(list):
        codify_file(f)
            
    h.write("\n}" * len(ns) + "\n");
    cpp.write("\n}" * len(ns) + "\n");
    
def read_folder_namespaces(folder):
    # read namespaces
    for dirpath, dirnames, filenames in os.walk(folder):
        for filename in [f for f in filenames if f.endswith(".xml")]:
            file = os.path.join(dirpath, filename)
            read_namespace(file)  
    
start = time.time()

# walk all sub folders
for dirname, dirnames, filenames in os.walk("./assets"):
    for d in dirnames:
        fulldir = os.path.join(dirname, d)
        library = os.path.join(fulldir, 'LIBRARY')
                
        if os.path.isdir(library):
            read_folder_namespaces(library)
            
# determine which classes are needed
for key in sorted(nsdic.keys()):
    parsexml(namespace(nsdic[key][0]), nsdic[key])

# forward declarations (h only)
for key in sorted(nsdic.keys()):
    forward_declare_namespace(key)

# codify classes (h and cpp)
for key in sorted(nsdic.keys()):
    codify_namespace(key)  

h.write(template_h_end)	
h.close()
cpp.close()

print "\n%.2f" % (time.time() - start), "seconds"